/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief GOPPRR XML to syntax tree transformer
 */

#ifndef __DSM_GOPPRRTRANSFORMER_H__
#define __DSM_GOPPRRTRANSFORMER_H__




#include "GOPPRRSyntaxTree.h"
#include <string>
#include "../GOPPRR/Graph.h"
#include "Error/ExceptionTypes.h"
#include <fstream>
#include <libxml++/nodes/element.h>
#include <map>
#include "SyntaxTransformer.h"
#include <libxml++/nodes/textnode.h>
#include "../GOPPRR/Project.h"
#include "../GOPPRR/Object.h"
#include "../GOPPRR/Port.h"
#include "../GOPPRR/Property.h"
#include "../GOPPRR/Role.h"
#include "../GOPPRR/Relationship.h"
#include "../GOPPRR/Connection.h"
#include "../GOPPRR/Binding.h"
#include "../GOPPRR/GraphicalContainer.h"


namespace oETCS { namespace UT { class CGOPPRRTransformer; }  }


namespace DSM {


/*!
 *  \brief XML syntax tree transformer for the GOPPRR meta meta model
 */
class CGOPPRRTransformer : public ::DSM::CSyntaxTransformer 
{
  public:
    /*!
     *  \brief default constructor
     *
     *  default constructor
     */
    CGOPPRRTransformer() throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CGOPPRRTransformer() throw();
    
    

    /*!
     *  \brief gets GOPRR syntax tree
     *
     *  Gets pointer to internal GOPRR syntax tree 
     *
     *  \return pointer to tree
     */
    ::DSM::CGOPPRRSyntaxTree* GetGOPRRTree() const throw();
    
    

    /*!
     *  \brief creates a GOPRR syntax tree
     *
     *  Creates a GOPPRR syntax tree from a XML model file while
     *  using an internalyl stored Schema file for validation.
     *  The create syntax tree is stored in m_pTree.
     *
     *  \param[in] ModelFileName file name of the XML model file
     *  \param[out] pRootGraph pointer to root graph, if one exists, else 0
     */
    void CreateSyntaxFromFile(const ::std::string & ModelFileName, GOPPRR::CGraph *& pRootGraph) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief creates a GOPRR syntax tree
     *
     *  Creates a GOPPRR syntax tree from a XML model file while
     *  using an internalyl stored Schema file for validation.
     *  The create syntax tree is stored in m_pTree.
     *
     *  \param[in] ModelFileName file stream of the XML model file
     *  \param[out] pRootGraph pointer to root graph, if one exists, else 0
     */
    void CreateSyntax(::std::fstream & ModelFileStream, GOPPRR::CGraph *& pRootGraph) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief creates a GOPRR syntax tree
     *
     *  Creates a GOPPRR syntax tree from a XML model file while
     *  using an internalyl stored Schema file for validation.
     *  The create syntax tree is stored in m_pTree.
     *
     *  \param[in] Model XML model as string
     *  \param[out] pRootGraph pointer to root graph, if one exists, else 0
     */
    void CreateSyntax(const ::std::string & Model, GOPPRR::CGraph *& pRootGraph) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief creates a GOPRR syntax tree
     *
     *  Creates a GOPPRR syntax tree from a XML model file while
     *  using an internalyl stored Schema file for validation.
     *  The create syntax tree is stored in m_pTree.
     *  
     *  \remark this methods works itereative and not recursive
     *
     *  \param[in] Model XML model as string
     *  \param[out] pRootGraph pointer to root graph, if one exists, else 0
     */
    void CreateSyntaxIterative(const ::std::string & Model, GOPPRR::CGraph *& pRootGraph) throw(DSM::Error::CException);
    
    


  protected:
    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CProperty object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreatePropertyIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CObject object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
    
     */
    static void CreateObjectIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a map
     *  of GOPRR::CGraph pointers as explosions from a XML element node.
     *  
     *  \param[in] pXMLElement:    pointer to current XML element <gopprr explosions> object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     *  \param[out] Explosions set of graph reference as current exlosions
     */
    static void CreateExplosionIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph > & Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty > & Properties, std::map< ::std::string, ::GOPPRR::CPort > & Ports, std::map< ::std::string, ::GOPPRR::CRole > & Roles, std::map< ::std::string, ::GOPPRR::CRelationship > & Relationships, std::map< ::std::string, ::GOPPRR::CGraph* >& Explosions) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CPort object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreatePortIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CRole object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreateRoleIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CRelationship object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreateRelationshipIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CGraph object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object/graph
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     *  \param[out] bRoot flag, if graph is root graph
     */
    static void CreateGraphIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph > & Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort > & Ports, std::map< ::std::string, ::GOPPRR::CRole > & Roles, std::map< ::std::string, ::GOPPRR::CRelationship > & Relationships, bool& bRoot) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CGraph object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object/graph
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     *  \param[out] bRoot flag, if graph is root graph
     */
    static void CreateGraph(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph > & Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort > & Ports, std::map< ::std::string, ::GOPPRR::CRole > & Roles, std::map< ::std::string, ::GOPPRR::CRelationship > & Relationships, bool& bRoot) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CGraph object from a XML element node,
     *  if it does not exist yet.
     *  
     *  \param[in] OID 	          object identifier of object
     *  \param[in] pXMLElement pointer current XML element
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     *  \param[out] bRoot flag, if graph is root graph
     */
    static void CreateGraph(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships, bool& bRoot) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CObject object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
    
     */
    static void CreateObject(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CObject object from a XML element node,
     *  if it does not exist yet.
     *  
     *  \param[in] OID 	          object identifier of object
     *  \param[in] pXMLElement pointer current XML element
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreateObject(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CPort object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreatePort(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CPort object from a XML element node,
     *  if it does not exist yet.
     *  
     *  \param[in] OID 	          object identifier of object
     *  \param[in] pXMLElement pointer current XML element
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreatePort(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CProperty object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreateProperty(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CProperty object from a XML element node,
     *  if it does not exist yet.
     *  
     *  \param[in] OID 	          object identifier of object
     *  \param[in] pXMLElement pointer current XML element
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreateProperty(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CRole object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreateRole(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CRole object from a XML element node,
     *  if it does not exist yet.
     *  
     *  \param[in] OID 	          object identifier of object
     *  \param[in] pXMLElement pointer current XML element
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreateRole(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CRelationship object from a XML element node.
     *  
     *  \param[in] pXMLElement pointer to current XML element object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreateRelationship(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a 
     *  GOPRR::CRelationship object from a XML element node,
     *  if it does not exist yet.
     *  
     *  \param[in] OID 	          object identifier of object
     *  \param[in] pXMLElement pointer current XML element
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     */
    static void CreateRelationship(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     * 
     *  Internal helper method for creating a map
     *  of GOPRR::CGraph pointers as explosions from a XML element node.
     *  
     *  \param[in] pXMLElement:    pointer to current XML element <gopprr explosions> object
     *  \param[inout] Graphs set of graphs in current project
     *  \param{inout] Objects:        set of objects in current project
     *  \param[inout] Properties set of properties in current project
     *  \param[inout] Ports set of ports in current project
     *  \param[inout] Roles set of roles in current project
     *  \param[inout] Relationships set of relationships in current project
     *  \param[out] Explosions set of graph reference as current exlosions
     */
    static void CreateExplosion(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph > & Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty > & Properties, std::map< ::std::string, ::GOPPRR::CPort > & Ports, std::map< ::std::string, ::GOPPRR::CRole > & Roles, std::map< ::std::string, ::GOPPRR::CRelationship > & Relationships, std::map< ::std::string, ::GOPPRR::CGraph* >& Explosions) throw(DSM::Error::CException);
    
    

    /*!
     *  \brief internal helper method
     *
     *  Internal helper method for finding a XML
     *  element by an XPath expression and its OID.
     *
     * \param[in] pXMLElement pointer to any XML element of docuement
     * \param[in] XPath XPath expression
     * \param[in] OID GOPPRR oid of element
     */
    static ::xmlpp::Element* FindElementByXPath(const ::xmlpp::Element * pXMLElement, const ::std::string & XPath, const ::std::string & OID) throw();
    
    


  private:
    /*!
     *  \brief XML schema for validation of GOPPRR model files
     */
    static ::std::string m_Schema;
    
    


// only include the following, if unit test framework is available
#ifdef __HAS_UNIT_TEST__

  friend class ::oETCS::UT::CGOPPRRTransformer;

#endif // #ifdef __HAS_UNIT_TEST__

}; // class CGOPPRRTransformer : public ::DSM::CSyntaxTransformer





} // namespace DSM

#endif // __ DSM_GOPPRRTRANSFORMER_H__
