/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#ifndef __GOPPRR_GRAPH_H__
#define __GOPPRR_GRAPH_H__




#include <string>
#include <map>
#include <vector>
#include "Error/ExceptionTypes.h"
#include "NonProperty.h"


namespace GOPPRR { struct CRole; } 
namespace GOPPRR { struct CObject; } 
namespace GOPPRR { struct CPort; } 
namespace GOPPRR { struct CRelationship; } 
namespace GOPPRR { struct CBinding; } 
namespace GOPPRR { struct CGraphicalContainer; } 


namespace GOPPRR {


/*!
 *  \brief graph class
 */
struct CGraph : public GOPPRR::CNonProperty 
{
    /*!
     *  \brief general constructor
     * 
     *  \param[in] OID object identifier
     *  \param[in] Type literal name of the type
     *  \param[in] ID identifier
     *  \param[in] Properties set of property references
     *  \param[in] BindingSet set of bindings
     *  \param[in] Containers set of graphical containers
     *  \param[in] ObjectSet set of objects
     *  \param[in] PortSet set of ports
     *  \param[in] RelationshipSet set of relationships
     *  \param[in] RoleSet set of roles
     */
    CGraph(const ::std::string& OID = "", const ::std::string& Type = "", const ::std::string& ID = "", const std::map< ::std::string, ::GOPPRR::CProperty* >& Properties = std::map< ::std::string, ::GOPPRR::CProperty* >(), const std::vector< ::GOPPRR::CBinding >& BindingSet = std::vector< ::GOPPRR::CBinding >(), const std::vector< ::GOPPRR::CGraphicalContainer >& Containers = std::vector< ::GOPPRR::CGraphicalContainer >(), const std::map< std::string, ::GOPPRR::CObject* >& ObjectSet = std::map< std::string, ::GOPPRR::CObject* >(), const std::map< std::string, ::GOPPRR::CPort* >& PortSet = std::map< std::string, ::GOPPRR::CPort* >(), const std::map< std::string, ::GOPPRR::CRelationship* >& RelationshipSet = std::map< std::string, ::GOPPRR::CRelationship* >(), const std::map< std::string, ::GOPPRR::CRole* >& RoleSet = std::map< std::string, ::GOPPRR::CRole* >()) throw();
    
    

    /*!
     *  \brief copy constructor
     *
     *  \param[in] Source source object to copy
     */
    CGraph(const CGraph& Source) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CGraph() throw();
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all roles connected to a certain object in a certain graph.
     *
     *  \param[in] pObject pointer to the object
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to roles 
     */
    ::std::map< ::std::string, GOPPRR::CRole* > Roles(GOPPRR::CObject * const pObject, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets certain roles connected to a certain object in a certain graph.
     *
     *  \param[in] pObject pointer to the object
     *  \param[in] Type role type 
     *  \param[in] bSubString if true, Type only must be a substring of role type
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to roles 
     */
    ::std::map< ::std::string, GOPPRR::CRole* > Roles(GOPPRR::CObject * const pObject, const ::std::string & Type, const bool & bSubString = false, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all roles connected to a certain port and object in a certain graph.
     *
     *  \param[in] pPort pointer to the port
     *  \param[in] pObject pointer to the object
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to roles 
     */
    ::std::map< ::std::string, GOPPRR::CRole* > Roles(GOPPRR::CObject * const pObject, GOPPRR::CPort * const pPort, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all roles connected to a certain relationship in a certain graph.
     *
     *  \param[in] pRelationship pointer to the relationship
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to roles 
     */
    ::std::map< ::std::string, GOPPRR::CRole* > Roles(GOPPRR::CRelationship * const pRelationship, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all certain types of roles connected to a certain port and object in a certain graph.
     *
     *  \param[in] pPort pointer to the port
     *  \param[in] pObjectt pointer to the object
     *  \param[in] Type role type 
     *  \param[in] bSubString if true, Type only must be a substring of role type
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to roles 
     */
    ::std::map< ::std::string, GOPPRR::CRole* > Roles(GOPPRR::CObject * const pObject, GOPPRR::CPort * const pPort, const ::std::string & Type, const bool & bSubString = false, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all certain types of roles connected to a certain relationship in a certain graph.
     *
     *  \param[in] pRelationship pointer to the relationship
     *  \param[in] Type role type 
     *  \param[in] bSubString if true, Type only must be a substring of role type
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to roles 
     */
    ::std::map< ::std::string, GOPPRR::CRole* > Roles(GOPPRR::CRelationship * const pRelationship, const ::std::string & Type, const bool & bSubString = false, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all certain types of roles connected to a certain another role in a certain graph.
     *  This skips the navigation via the related relationship.
     *
     *  \param[in] pRole pointer to the role
     *  \param[in] Type role type 
     *  \param[in] bSubString if true, Type only must be a substring of role type
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to roles 
     */
    ::std::map< ::std::string, GOPPRR::CRole* > Roles(GOPPRR::CRole * const pRole, const ::std::string & Type, const bool & bSubString = false, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all relationships connected to a certain role in a certain graph.
     *
     *  \param[in] pRole pointer to the role
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to relationships
     */
    ::std::map< ::std::string, GOPPRR::CRelationship* > Relationships(GOPPRR::CRole * const pRole, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets certain types of relationships connected to a certain role in a certain graph.
     *
     *  \param[in] pRole pointer to the role
     *  \param[in] Type relationship type 
     *  \param[in] bSubString if true, Type only must be a substring of relationship type
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to relationships
     */
    ::std::map< ::std::string, GOPPRR::CRelationship* > Relationships(GOPPRR::CRole * const pRole, const ::std::string & Type, const bool & bSubString = false, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all ports connected to a certain role in a certain graph.
     *
     *  \param[in] pRole pointer to the role
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to ports
     */
    ::std::map< ::std::string, GOPPRR::CPort* > Ports(GOPPRR::CRole * const pRole, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all ports connected to a certain object in a certain graph.
     *
     *  \param[in] pObject pointer to the object
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to ports
     */
    ::std::map< ::std::string, GOPPRR::CPort* > Ports(GOPPRR::CObject * const pObject, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets certain types of ports connected to a certain object in a certain graph.
     *
     *  \param[in] pObject pointer to the object
     *  \param[in] Type port type 
     *  \param[in] bSubString if true, Type only must be a substring of port type
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to ports
     */
    ::std::map< ::std::string, GOPPRR::CPort* > Ports(GOPPRR::CObject * const pObject, const ::std::string & Type, const bool & bSubString = false, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all objects connected to a certain port and role in a certain graph.
     *
     *  \param[in] pPort pointer to the port
     *  \param[in] pRole           pointer to the role
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to objects
     */
    ::std::map< ::std::string, GOPPRR::CObject* > Objects(GOPPRR::CPort * const pPort, GOPPRR::CRole * const pRole, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all objects connected to a certain role in a certain graph.
     *
     *  \param[in] pRole pointer to the role
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to objects
     */
    ::std::map< ::std::string, GOPPRR::CObject* > Objects(GOPPRR::CRole * const pRole, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets certain types of objects connected to a certain role in a certain graph.
     *
     *  \param[in] pRole pointer to the role
     *  \param[in] Type object type 
     *  \param[in] bSubString if true, Type only must be a substring of object type
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to objects
     */
    ::std::map< ::std::string, GOPPRR::CObject* > Objects(GOPPRR::CRole * const pRole, const ::std::string & Type, const bool & bSubString = false, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets certain types of objects in a certain graph.
     *
     *  \param[in] Type object type 
     *  \param[in] bSubString if true, Type only must be a substring of object type
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to objects
     */
    ::std::map< ::std::string, GOPPRR::CObject* > Objects(const ::std::string & Type, const bool & bSubString = false, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all objects which graphical contain another one in a certain graph.
     *
     *  \param[in] pObject pointer to the object
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to objects
     */
    ::std::map< ::std::string, GOPPRR::CObject* > Container(GOPPRR::CObject * const pObject, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets all objects graphical contained by a another one in a certain graph.
     *
     *  \param[in] pObject pointer to the object
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to objects
     */
    ::std::map< ::std::string, GOPPRR::CObject* > Contained(GOPPRR::CObject * const pObject, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief assignment operator
     *
     *  \param[in] Source object instance to assign from
     */
    CGraph & operator=(const CGraph& Source) throw();
    
    

    /*!
     *  \brief set of bindings in a graph
     *
     *  Set of bindings in a graph.
     */
    ::std::vector< GOPPRR::CBinding > m_BindingSet;
    
    

    /*!
     *  \brief set of objects in a graph
     * 
     *  Set of objects in a graph indexed by
     *  their OID.
     */
    ::std::map< std::string,GOPPRR::CObject* > m_ObjectSet;
    
    

    /*!
     *  \brief set of ports
     *  
     *  Set of ports indexed by
     *  their OID.
     */
    ::std::map< std::string,GOPPRR::CPort* > m_PortSet;
    
    

    /*!
     *  \brief set of relationships in a graph
     *
     *  Set of relationships in a graph indexed
     *  by their OID
     */
    ::std::map< std::string,GOPPRR::CRelationship* > m_RelationshipSet;
    
    

    /*!
     *  \brief role set of a graph
     *
     *  Role set of a graph indexed by
     *  their OID.
     */
    ::std::map< std::string,GOPPRR::CRole* > m_RoleSet;
    
    

    /*!
     *  \brief graphical containments in a graph
     */
    ::std::vector< GOPPRR::CGraphicalContainer > m_Containers;
    
    


}; // struct CGraph : public GOPPRR::CNonProperty





} // namespace GOPPRR

#endif // __ GOPPRR_GRAPH_H__
