/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief driver machine classes for observer usage
 */

#include "DriverMachineInterfaceMOC.h"
#include "EVCStateMachine.h"




namespace oETCS {

namespace DF {


CDMIQWidget::CDMIQWidget(oETCS::DF::CEVCStateMachine * const pStateMachine, QWidget * pWidgetParent, const bool& bSimAdaptor)  throw()  
:QWidget(pWidgetParent),
 CDMIObserver(pStateMachine),
 m_pAdaptor(nullptr),
 m_Layout(this),
 m_Labels(0),
 m_InputWidgets(0),
 m_OutputWidgets(0),
 m_pLastSubject(0),
 m_InputButtons(0),
 m_Mutex(QMutex::Recursive)
{
  // Bouml preserved body begin 000C3302
    bool                bConnected(true);
    QDBusConnection     Connection = QDBusConnection::sessionBus();
    
    
    // attach this observer to the EVC state machine
    m_pStateMachine->AttachObserver(this);
    
    // connect slots and signal
    QObject::connect(this, SIGNAL(Updated(void)), this, SLOT(UpdateSlot(void)), Qt::AutoConnection);
    
    // set size policy of widget
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    
    // call initial update to create input and output elements
    this->Update();
    
    // check, if simulation adaptor should be used
    if (bSimAdaptor)
    {
        try
        {
            // create adaptor object
            m_pAdaptor = new ::ADMI(this);
            
            // register all adaptors
            bConnected &= Connection.registerService("oETCS.DF.IDMI");
            bConnected &= Connection.registerObject("/oETCS/DMI", this);
        } // 
        catch (const ::std::bad_alloc& Exception)
        {
            // only print error, cannot do more in constructor
            ::std::cerr << Exception.what() << ::std::endl;
            
        } // catch (const ::std::bad_alloc& Exception)
        
        // check, if DMI adaptor could be registered successfully
        if (not bConnected)
        {
            // print error
            ::std::cerr << "DMI adaptor was not registered successfully on D-Bus" << ::std::endl;
            
        } // if (not bConnected)
        
    } // if (bSimAdaptor)
  // Bouml preserved body end 000C3302

} //  CDMIQWidget::CDMIQWidget()  throw() 




CDMIQWidget::~CDMIQWidget()  throw()  
{
  // Bouml preserved body begin 000C3382
    decltype (m_InputWidgets.begin())                       iw(0);
    decltype (m_OutputWidgets.begin())                      ow(0);
    decltype (m_Labels.begin())                             l(0);
    decltype (m_InputButtons.begin())                       ib(0);
    
    
    
    // remove all input widgets from layout and delete them
    for (iw = m_InputWidgets.begin(); iw != m_InputWidgets.end(); iw++)
    {
        // remove current widget from layout
        m_Layout.removeWidget(*iw);

        // delete widget
        delete *iw;

    } // for (iw = m_InputWidgets.begin(); iw != m_InputWidgets.end(); iw++)

    // clear input widget vector
    m_InputWidgets.clear();

    // remove all output widgets from layout and delete them
    for (ow = m_OutputWidgets.begin(); ow != m_OutputWidgets.end(); ow++)
    {
        // remove current widget from layout
        m_Layout.removeWidget(*ow);

        // delete widget
        delete *ow;

    } // for (ow = m_OutputWidgets.begin(); ow != m_OutputWidgets.end(); ow++)

    // clear output widget vector
    m_OutputWidgets.clear();

    // remove all labels from layout and delete them
    for (l = m_Labels.begin(); l != m_Labels.end(); l++)
    {
        // remove current label from layout
        m_Layout.removeWidget(*l);

        // delete label
        delete *l;

    } // for (l = m_Labels.begin(); l != m_Labels.end(); l++)

    // clear label vector
    m_Labels.clear();

    // remove all input buttons from layout and delete them
    for (ib = m_InputButtons.begin(); ib != m_InputButtons.end(); ib++)
    {
        // remove current button from layout
        m_Layout.removeWidget(*ib);

        // delete input button
        delete *ib;

    } // for (ib = m_InputButtons.begin(); ib != m_InputButtons.end(); ib++)

    // clear input button vector
    m_InputButtons.clear();
    
    // delete adaptor
    delete (m_pAdaptor);
  // Bouml preserved body end 000C3382

} // CDMIQWidget::~CDMIQWidget()  throw() 




void CDMIQWidget::Update()  throw()  
{
  // Bouml preserved body begin 000CBF02
    // just emit signal for update here to start slot in GUI/QApplication thread
    emit Updated();
  // Bouml preserved body end 000CBF02

} // void CDMIQWidget::Update()  throw() 




void CDMIQWidget::Detached()  throw()  
{
  // Bouml preserved body begin 000CD982
    // close this widget
    this->close();
  // Bouml preserved body end 000CD982

} // void CDMIQWidget::Detached()  throw() 




void CDMIQWidget::DataEntered()  throw()  
{
  // Bouml preserved body begin 000CD882
    decltype (m_InputButtons.begin())                       ib(0);
    decltype (CDMIInput::m_AllowedBoolInputs.begin())       abi;
    decltype (CDMIInput::m_AllowedDoubleInputs.begin())     adi;
    decltype (CDMIInput::m_AllowedIntInputs.begin())        aii;
    decltype (CDMIInput::m_AllowedStringInputs.begin())     asi;
    QObject*                                                pSender(0);
    QString                                                 AllowedValues;
    QMessageBox                                             ErrorMessage(this);
    ::std::string                                           StringInput;
    unsigned int                                            uiIndex(0);
    double                                                  dDoubleInput(0.0);
    int                                                     iIntInput(0);
    bool                                                    bFound(false);
    bool                                                    bBoolInput(false);
    
    
    
    
    // set general message box properties
    ErrorMessage.setWindowModality(Qt::NonModal);
    ErrorMessage.setText(tr("Error while setting input: value not allowed"));
    ErrorMessage.setStandardButtons(QMessageBox::Ok);
    ErrorMessage.setWindowTitle(tr("Input value not allowed"));
    ErrorMessage.setIcon(QMessageBox::Critical);
    
    
    // lock mutex
    m_Mutex.lock();
    
    // initial try to get the sender for this slot
    pSender = this->sender();
    
    // check, if sender was a push button, otherwise ignore the signal (should never happen anyway)
    if (dynamic_cast< QPushButton* >(pSender) != 0)
    {
        // find the pressed button in vector
        for (uiIndex = 0, ib = m_InputButtons.begin(); ib != m_InputButtons.end() && not bFound; uiIndex++, ib++)
        {
            // compare current button with sender
            bFound = (*ib == pSender);
            
        } // for (uiIndex = 0, ib = m_InputButtons.begin(); ib != m_InputButtons.end() && not bFound; uiIndex++, ib++)
        
        // check, if sender button was found (otherwise just ignore the signal)
        if (bFound)
        {
            // decrement index once to gain correct one
            uiIndex--;
            
            // switch the type of the corresponding input value
            switch (m_pLastSubject->m_Inputs[uiIndex]->m_eType)
            {
                case ::oETCS::DF::BOOL:
                    // get boolean input from check box
                    bBoolInput = static_cast< QCheckBox* >(m_InputWidgets[uiIndex])->checkState() == 0 ? false : true;
                    
                    // check is new value is allowed
                    if (not CDMIQWidget::IsPartOf(bBoolInput, m_pLastSubject->m_Inputs[uiIndex]->m_AllowedBoolInputs))
                    {
                        // convert vector of allowed values to string
                        for (abi = m_pLastSubject->m_Inputs[uiIndex]->m_AllowedBoolInputs.begin(); abi != m_pLastSubject->m_Inputs[uiIndex]->m_AllowedBoolInputs.end(); abi++)
                        {
                            // add current allowed value to string
                            AllowedValues += QString(*abi ? "true" : "false")  + QString("; ");
                            
                        } // for (abi = m_pLastSubject->m_Inputs[uiIndex]->m_AllowedBoolInputs.begin(); abi != m_pLastSubject->m_Inputs[uiIndex]->m_AllowedBoolInputs.end(); abi++)
                        
                        // set certain message box properties
                        ErrorMessage.setInformativeText(tr("Allowed values: ") + AllowedValues);
                        
                        // show error message box non-modal
                        ErrorMessage.exec();
                        
                        
                    } // if (not CDMIQWidget::IsPartOf(bBoolInput, m_pLastSubject->m_Inputs[uiIndex]->m_AllowedBoolInputs))
                    else
                    {
                        // set value in DMI input
                        m_pLastSubject->m_Inputs[uiIndex]->m_bBool = bBoolInput;

                    } // else

                    break;


                case ::oETCS::DF::DOUBLE:
                    // get boolean input from double spin box
                    dDoubleInput = static_cast< QDoubleSpinBox* >(m_InputWidgets[uiIndex])->value();

                    // check is new value is allowed
                    if (not CDMIQWidget::IsPartOf(dDoubleInput, m_pLastSubject->m_Inputs[uiIndex]->m_AllowedDoubleInputs))
                    {
                        // convert vector of allowed values to string
                        for (adi = m_pLastSubject->m_Inputs[uiIndex]->m_AllowedDoubleInputs.begin(); adi != m_pLastSubject->m_Inputs[uiIndex]->m_AllowedDoubleInputs.end(); adi++)
                        {
                            // add current allowed value to string
                            AllowedValues += QString::number(*adi)  + QString("; ");

                        } // for (adi = m_pLastSubject->m_Inputs[uiIndex]->m_AllowedDoubleInputs.begin(); adi != m_pLastSubject->m_Inputs[uiIndex]->m_AllowedDoubleInputs.end(); adi++)

                        // set certain message box properties
                        ErrorMessage.setInformativeText(tr("Allowed values: ") + AllowedValues);

                        // show error message box non-modal
                        ErrorMessage.exec();


                    } // if (not CDMIQWidget::IsPartOf(dDoubleInput, m_pLastSubject->m_Inputs[uiIndex]->m_AllowedDoubleInputs))
                    else
                    {
                        // set value in DMI input
                        m_pLastSubject->m_Inputs[uiIndex]->m_dDouble = dDoubleInput;

                    } // else

                    break;


                case ::oETCS::DF::DOUBLE_ARRAY:
                    
                    // not (yet) implemented
                    
                    break;


                case ::oETCS::DF::INT:
                    // get integer input from spin box
                     iIntInput = static_cast< QSpinBox* >(m_InputWidgets[uiIndex])->value();

                     // check is new value is allowed
                     if (not CDMIQWidget::IsPartOf(iIntInput, m_pLastSubject->m_Inputs[uiIndex]->m_AllowedIntInputs))
                     {
                         // convert vector of allowed values to string
                         for (aii = m_pLastSubject->m_Inputs[uiIndex]->m_AllowedIntInputs.begin(); aii != m_pLastSubject->m_Inputs[uiIndex]->m_AllowedIntInputs.end(); aii++)
                         {
                             // add current allowed value to string
                             AllowedValues += QString::number(*aii)  + QString("; ");

                         } // for (aii = m_pLastSubject->m_Inputs[uiIndex]->m_AllowedIntInputs.begin(); aii != m_pLastSubject->m_Inputs[uiIndex]->m_AllowedIntInputs.end(); aii++)

                         // set certain message box properties
                         ErrorMessage.setInformativeText(tr("Allowed values: ") + AllowedValues);

                         // show error message box non-modal
                         ErrorMessage.exec();


                     } // if (not CDMIQWidget::IsPartOf(iIntInput, m_pLastSubject->m_Inputs[uiIndex]->m_AllowedIntInputs))
                     else
                     {
                         // set value in DMI input
                         m_pLastSubject->m_Inputs[uiIndex]->m_iInt = iIntInput;

                     } // else
                  
                    break;


                case ::oETCS::DF::STRING:
                    // get string input from spin box
                     StringInput = static_cast< QLineEdit* >(m_InputWidgets[uiIndex])->text().toStdString();

                     // check is new value is allowed
                     if (not CDMIQWidget::IsPartOf(StringInput, m_pLastSubject->m_Inputs[uiIndex]->m_AllowedStringInputs))
                     {
                         // convert vector of allowed values to string
                         for (asi = m_pLastSubject->m_Inputs[uiIndex]->m_AllowedStringInputs.begin(); asi != m_pLastSubject->m_Inputs[uiIndex]->m_AllowedStringInputs.end(); asi++)
                         {
                             // add current allowed value to string
                             AllowedValues += QString((*asi).c_str()) + QString("; ");

                         } // for (asi = m_pLastSubject->m_Inputs[uiIndex]->m_AllowedStringInputs.begin(); asi != m_pLastSubject->m_Inputs[uiIndex]->m_AllowedStringInputs.end(); asi++)

                         // set certain message box properties
                         ErrorMessage.setInformativeText(tr("Allowed values: ") + AllowedValues);

                         // show error message box non-modal
                         ErrorMessage.exec();


                     } // if (not CDMIQWidget::IsPartOf(iIntInput, m_pLastSubject->m_Inputs[uiIndex]->m_AllowedIntInputs))
                     else
                     {
                         // set value in DMI input
                         m_pLastSubject->m_Inputs[uiIndex]->m_String = StringInput;

                     } // else
                   
                    break;
                
            } // switch (m_pLastSubject->m_Inputs[uiIndex]->m_eType)
            
        } // if (bFound) 
    
    } // if (dynamic_cast< QPushButton* >(pSender) != 0)
    
    // unlock mutex
    m_Mutex.unlock();
  // Bouml preserved body end 000CD882

} // void CDMIQWidget::DataEntered()  throw() 




void CDMIQWidget::UpdateSlot()  throw()  
{
  // Bouml preserved body begin 000E4282
    ::oETCS::DF::CDMISubject*                               pSubject(nullptr);
    decltype (CDMISubject::m_Inputs.begin())                i(0);
    decltype (CDMISubject::m_Outputs.begin())               o(0);
    decltype (m_InputWidgets.begin())                       iw(0);
    decltype (m_OutputWidgets.begin())                      ow(0);
    decltype (m_Labels.begin())                             l(0);
    decltype (m_InputButtons.begin())                       ib(0);
    decltype (CDMIOutput::m_DoubleArrayMessage().begin())   da(0);
    QMessageBox                                             ErrorMessage(this);
    unsigned int                                            uiRow(0);
    bool                                                    bSubjectChanged(false);
    
    
    
    // lock mutex
    m_Mutex.lock();
    
    // get subject from state machine
    pSubject = m_pStateMachine->GetCurrentDMI();
    
    // check, if pointer to current subject is valid
    if (pSubject != nullptr)
    {
        // check, if only values but not subject itself has changed
        bSubjectChanged = (pSubject != m_pLastSubject);
        
        try
        {
            // check, if subject has changed
            if (bSubjectChanged)
            {
                // remove all input widgets from layout and delete them
                for (iw = m_InputWidgets.begin(); iw != m_InputWidgets.end(); iw++)
                {
                    // remove current widget from layout
                    m_Layout.removeWidget(*iw);

                    // delete widget
                    delete *iw;

                } // for (iw = m_InputWidgets.begin(); iw != m_InputWidgets.end(); iw++)

                // clear input widget vector
                m_InputWidgets.clear();

                // remove all output widgets from layout and delete them
                for (ow = m_OutputWidgets.begin(); ow != m_OutputWidgets.end(); ow++)
                {
                    // remove current widget from layout
                    m_Layout.removeWidget(*ow);

                    // delete widget
                    delete *ow;

                } // for (ow = m_OutputWidgets.begin(); ow != m_OutputWidgets.end(); ow++)

                // clear output widget vector
                m_OutputWidgets.clear();

                // remove all labels from layout and delete them
                for (l = m_Labels.begin(); l != m_Labels.end(); l++)
                {
                    // remove current label from layout
                    m_Layout.removeWidget(*l);

                    // delete label
                    delete *l;

                } // for (l = m_Labels.begin(); l != m_Labels.end(); l++)

                // clear label vector
                m_Labels.clear();

                // remove all input buttons from layout and delete them
                for (ib = m_InputButtons.begin(); ib != m_InputButtons.end(); ib++)
                {
                    // remove current button from layout
                    m_Layout.removeWidget(*ib);

                    // delete input button
                    delete *ib;

                } // for (ib = m_InputButtons.begin(); ib != m_InputButtons.end(); ib++)

                // clear input button vector
                m_InputButtons.clear();
                
                // process all outputs of current subject
                for (uiRow = 0, o = pSubject->m_Outputs.begin(); o != pSubject->m_Outputs.end(); uiRow++, o++)
                {
                    // create QLabel
                    m_Labels.push_back(new QLabel((*o)->m_Label.c_str(), this));

                    // add current label to grid layout
                    m_Layout.addWidget(m_Labels[uiRow], uiRow, 0);

                    // create QLineEdit as general output
                    m_OutputWidgets.push_back(new QLineEdit(this));

                    // disable editing in new object
                    m_OutputWidgets[uiRow]->setEnabled(false);

                    // add current input widget to grid layout
                    m_Layout.addWidget(m_OutputWidgets[uiRow], uiRow, 1);
                    
                } // for (uiRow = 0, o = pSubject->m_Outputs.begin(); o != pSubject->m_Outputs.end(); uiRow++, o++)


                // process all inputs of current subject
                for (uiRow = 0, i = pSubject->m_Inputs.begin(); i != pSubject->m_Inputs.end(); uiRow++, i++)
                {
                    // create QLabel
                    m_Labels.push_back(new QLabel((*i)->m_Label.c_str(), this));

                    // add current label to grid layout
                    m_Layout.addWidget(m_Labels[uiRow + m_OutputWidgets.size()], uiRow, 2);
                  

                    // switch type of current input object for concrete widget creation
                    switch ((*i)->m_eType)
                    {
                        case ::oETCS::DF::BOOL:
                            // create check box 
                            m_InputWidgets.push_back(new QCheckBox(this));

                            break;


                        case ::oETCS::DF::DOUBLE:
                            // create double spin box
                            m_InputWidgets.push_back(new QDoubleSpinBox(this));

                            // set minimum and maximum
                            static_cast < ::QDoubleSpinBox* >(*(m_InputWidgets.end() - 1))->setMinimum(0.0);
                            static_cast < ::QDoubleSpinBox* >(*(m_InputWidgets.end() - 1))->setMaximum(10000.0);
                            
                            break;


                        case ::oETCS::DF::DOUBLE_ARRAY:
                            // create list view
                            m_InputWidgets.push_back(new QListView(this));

                            break;


                        case ::oETCS::DF::INT:
                            // create spin box
                            m_InputWidgets.push_back(new QSpinBox(this));
                            
                            // set minimum and maximum
                            static_cast < ::QSpinBox* >(*(m_InputWidgets.end() - 1))->setMinimum(0);
                            static_cast < ::QSpinBox* >(*(m_InputWidgets.end() - 1))->setMaximum(10000);

                            break;


                        case ::oETCS::DF::STRING:
                            // create line edit
                            m_InputWidgets.push_back(new QLineEdit(this));

                            break;

                    } // switch ((*i)->m_eType)

                    // add current input widget to grid layout
                    m_Layout.addWidget(m_InputWidgets[uiRow], uiRow, 3);

                    // create input push button
                    m_InputButtons.push_back(new QPushButton("Enter Data", this));
                    
                    // add current input button to grid layout
                    m_Layout.addWidget(m_InputButtons[uiRow], uiRow, 4);

                    // connect clicked signal of button with slot
                    QObject::connect(m_InputButtons[uiRow], SIGNAL(clicked(bool)), this, SLOT(DataEntered(void)), Qt::AutoConnection);

                } // for (uiRow = 0, i = pSubject->m_Inputs.begin(); i != pSubject->m_Inputs.end(); uiRow++, i++)

            } // if (bSubjectChanged)

            
            // process all outputs of current subjects (again) to set the input values from subject
            for (uiRow = 0, o = pSubject->m_Outputs.begin(); o != pSubject->m_Outputs.end(); uiRow++, o++)
            {
                // set visibility flag from current output
                m_OutputWidgets[uiRow]->setVisible((*o)->m_bVisible());
                m_Labels[uiRow]->setVisible((*o)->m_bVisible());

                // switch type of current output of subject for setting the output value
                switch ((*o)->m_eType)
                {
                    case ::oETCS::DF::BOOL:
                        // set value of QLineEdit from boolean
                        static_cast< QLineEdit* >(m_OutputWidgets[uiRow])->setText((*o)->m_bBoolMessage() ? "true" : "false");

                        break;


                    case ::oETCS::DF::DOUBLE:
                        // set value of QLineEdit from double
                        static_cast< QLineEdit* >(m_OutputWidgets[uiRow])->setText(QString::number((*o)->m_dDoubleMessage()));

                        break;


                    case ::oETCS::DF::DOUBLE_ARRAY:
                        // set value of QLineEdit from double
                        for (da = (*o)->m_DoubleArrayMessage().begin(); da != (*o)->m_DoubleArrayMessage().end(); da++)
                        {
                            // copy current double value
                            static_cast< QLineEdit* >(m_OutputWidgets[uiRow])->setText(static_cast< QLineEdit* >((*(m_OutputWidgets.end() - 1)))->text()+ QString::number(*da)+ "; ");

                        } // for (da = (*o)->m_DoubleArrayMessage().begin(); da != (*o)->m_DoubleArrayMessage().end(); da++)

                        break;


                    case ::oETCS::DF::INT:
                        // set value of QLineEdit from integer
                        static_cast< QLineEdit* >(m_OutputWidgets[uiRow])->setText(QString::number((*o)->m_iIntMessage()));

                        break;


                    case ::oETCS::DF::STRING:
                        // set value of QLineEdit from string
                        static_cast< QLineEdit* >(m_OutputWidgets[uiRow])->setText((*o)->m_StringMessage().c_str());

                        break;

                } // switch ((*o)->m_eType)

            } // for (uiRow = 0, o = pSubject->m_Outputs.begin(); o != pSubject->m_Outputs.end(); uiRow++, o++)
            
            
            // process all inputs of current subject (again) to set visibility flag
            for (uiRow = 0, i = pSubject->m_Inputs.begin(); i != pSubject->m_Inputs.end(); uiRow++, i++)
            {
                // set visibility flag from current output
                m_InputWidgets[uiRow]->setVisible((*i)->m_bVisible());
                m_Labels[uiRow + m_OutputWidgets.size()]->setVisible((*i)->m_bVisible());
                m_InputButtons[uiRow]->setVisible((*i)->m_bVisible());
                
            } // for (uiRow = 0, i = pSubject->m_Inputs.begin(); i != pSubject->m_Inputs.end(); uiRow++, i++)
            

            // store pointer to current subject
            m_pLastSubject = pSubject;

        } // try
        catch (const ::std::bad_alloc& Exception)
        {
            // reset subject storage
            m_pLastSubject = 0;
            
            // set message box
            ErrorMessage.setWindowModality(Qt::NonModal);
            ErrorMessage.setText(tr("Error while allocation of memory for GUI elements.\nThe GUI representation might not be correct!"));
            ErrorMessage.setStandardButtons(QMessageBox::Ok);
            ErrorMessage.setWindowTitle(tr("Not enough memory"));
            ErrorMessage.setInformativeText(tr("Unfortunately there is nothing you can do about it."));
            ErrorMessage.setIcon(QMessageBox::Critical);
            
            // show message box non-model
            ErrorMessage.exec();

        } // catch (const ::std::bad_alloc& Exception)
    
    } // if (pSubject != nullptr)

    // unlock mutex
    m_Mutex.unlock();

  // Bouml preserved body end 000E4282

} // void CDMIQWidget::UpdateSlot()  throw() 




QStringList CDMIQWidget::GetInputs()   
{
  // Bouml preserved body begin 000F1702
    QStringList                         Inputs;
    decltype (m_Labels)                 Labels(m_Labels);
    decltype (m_InputWidgets)           InputWidgets(m_InputWidgets);
    unsigned int                        x(0);
    const unsigned int                  NUMBER_INPUTS(m_InputWidgets.size());
    const unsigned int                  NUMBER_OUTPUTS(m_OutputWidgets.size());
    
    
    
    // get all labels for input (second)
    for (x = NUMBER_OUTPUTS; x < (NUMBER_INPUTS + NUMBER_OUTPUTS); x++)
    {
        // check, if current input field is visible
        if (InputWidgets[x - NUMBER_OUTPUTS]->isVisible())
        {
            // add current label value to string list
            Inputs.append(Labels[x]->text());
            
            
            #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
            // DEBUG
            ::std::cout << "DEBUG: GetInputs() -> input #" << x << " \"" << Labels[x]->text().toStdString() << "\"" << ::std::endl;
            #endif 

        } // if (InputWidgets[x - NUMBER_OUTPUTS]->isVisible())
    
    } // for (x = NUMBER_OUTPUTS; x < (NUMBER_INPUTS + NUMBER_OUTPUTS); x++)
    
    #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "DEBUG: GetInputs() -> number of inputs in list " << Inputs.size() << ::std::endl;
    #endif
    
    // return inputs
    return (Inputs);
  // Bouml preserved body end 000F1702

} // ::QStringList CDMIQWidget::GetInputs()  throw() 




::QStringList CDMIQWidget::GetOutputs()   
{
  // Bouml preserved body begin 000F3082
    QStringList                         Outputs;
    unsigned int                        x(0);
    const unsigned int                  NUMBER_OUTPUTS(m_OutputWidgets.size());

    

    // lock mutex
    m_Mutex.lock();
    
    // get all labels for output (first)
    for (x = 0; x < NUMBER_OUTPUTS; x++)
    {
        // check, if current output is visible
        if (m_OutputWidgets[x]->isVisible())
        {
            // add current label value to string list
            Outputs.append(m_Labels[x]->text());

        } // if (m_OutputWidgets[x]->isVisible())

    } // for (x = 0; x < NUMBER_OUTPUTS; x++)
    
    // unlock mutex
    m_Mutex.unlock();
    
    // return outputs
    return (Outputs);
  // Bouml preserved body end 000F3082

} // ::QStringList CDMIQWidget::GetOutputs()  throw() 




QString CDMIQWidget::GetOutputValue(const QString & OutputName)   
{
  // Bouml preserved body begin 000F3102
    QString                     Value;
    decltype (m_Labels.begin()) x;
    int                         p(0);
    int                         iPosition(-1);


    
    #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "DEBUG: GetOutputValue() -> started" << ::std::endl;
    #endif

    // lock mutex
    m_Mutex.lock();

    // try to find output field in labels
    for (x = m_Labels.begin(), p = 0; x != m_Labels.end() - m_InputWidgets.size() && iPosition == -1; x++, p++)
    {
        // check, if current label corresponds to output field and is visible
        if ((*x)->text() == OutputName && (*x)->isVisible())
        {
            // store position
            iPosition = p;

        } // if ((*x)->text() == OutputName && (*x)->isVisible())

    } // for (x = m_Labels.begin(), p = 0; x != m_Labels.end() - m_InputWidgets.size() && iPosition == -1; x++, p++)


    // check, if output field could be found
    if (iPosition != -1)
    {
        // get output field value
        Value = static_cast< ::QLineEdit* >(m_OutputWidgets[iPosition])->text();

    } // if (iPosition != -1)
    else
    {
        // print warning
        ::std::cerr << "warning in CDMIQWidget::GetOutputValue() in " << __FILE__ << ":" << __LINE__ << " -> output field \"" << OutputName.toStdString() << "\" not found" << ::std::endl;
        
    } // else

    // unlock mutex
    m_Mutex.unlock();
    
    #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "DEBUG: GetOutputValue() -> finished with value \"" << Value.toStdString() << "\"" << ::std::endl;
    #endif

    // return output value
    return (Value);
  // Bouml preserved body end 000F3102

} // ::QString CDMIQWidget::GetOutputValue()  throw() 




bool CDMIQWidget::HasInput()   
{
  // Bouml preserved body begin 000F3182
    bool            bHasInput(false);
    
    
    // get list of outputs and evaluate size
    bHasInput = !(this->GetInputs().empty());
    
    // return input flag
    return (bHasInput);
  // Bouml preserved body end 000F3182

} // bool CDMIQWidget::HasInput()  throw() 




void CDMIQWidget::SetBooleanValue(const QString & InputName, bool bValue)   
{
  // Bouml preserved body begin 000F3202
    decltype (m_Labels.begin()) x;
    int                         p(0);
    int                         iPosition(-1);
    
    
    
    #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "DEBUG: started CDMIQWidget::SetBooleanValue()" << ::std::endl;
    #endif
    
    // lock mutex
    m_Mutex.lock();
    
    // try to find input field in labels
    for (x = m_Labels.begin() + m_OutputWidgets.size(), p = 0; x != m_Labels.end() && iPosition == -1; x++, p++)
    {
        // check, if current label corresponds to input field
        if ((*x)->text() == InputName)
        {
            // store position
            iPosition = p;
            
        } // if ((*x)->text() == InputName)
        
    } // for (x = m_Labels.begin() + m_OutputWidgets.size(), p = 0; x != m_Labels.end() && iPosition == -1; x++, p++)
    
    // check, if input field was found
    if (iPosition!= -1)
    {
        // check, if input widget is a QCheckBox
        if (dynamic_cast< ::QCheckBox* >(m_InputWidgets[iPosition]) != nullptr)
        {
            // check, if input field is visible
            if (static_cast< ::QCheckBox* >(m_InputWidgets[iPosition])->isVisible())
            {
                // set input field (boolean)
                static_cast< ::QCheckBox* >(m_InputWidgets[iPosition])->setChecked(bValue);

                // press related input button to activate update
                m_InputButtons[iPosition]->click();
            
            } // if (static_cast< ::QCheckBox* >(m_InputWidgets[iPosition])->isVisible())
            else
            {
                // print warning
                ::std::cerr << "warning in CDMIQWidget::SetBooleanValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" is currently not visible" << ::std::endl;
                
            } // else


        } // if (dynamic_cast< ::QCheckBox* >(m_InputWidgets[iPosition]) != nullptr)
        else
        {
            // print warning
            ::std::cerr << "warning in CDMIQWidget::SetBooleanValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" is not a boolean field" << ::std::endl;

        } // else    
        
    } // if (iPosition != -1)
    else
    {
        // print warning
        ::std::cerr << "warning in CDMIQWidget::SetBooleanValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" not found" << ::std::endl;
        
    } // else
    
    // unlock mutex
    m_Mutex.unlock();
    
    #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "DEBUG: finished CDMIQWidget::SetBooleanValue()" << ::std::endl;
    #endif
  // Bouml preserved body end 000F3202

} // void CDMIQWidget::SetBooleanValue()  throw() 




void CDMIQWidget::SetDoubleValue(const QString & InputName, double dValue)   
{
  // Bouml preserved body begin 000F3282
    decltype (m_Labels.begin()) x;
    int                         p(0);
    int                         iPosition(-1);
    

    
    #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "DEBUG: started CDMIQWidget::SetDoubleValue()" << ::std::endl;
#endif
    
    // lock mutex
    m_Mutex.lock();
    
    // try to find input field in labels
    for (x = m_Labels.begin() + m_OutputWidgets.size(), p = 0; x != m_Labels.end() && iPosition == -1; x++, p++)
    {
        // check, if current label corresponds to input field
        if ((*x)->text() == InputName)
        {
            // store position
            iPosition = p;
            
        } // if ((*x)->text() == InputName)
        
    } // for (x = m_Labels.begin() + m_OutputWidgets.size(), p = 0; x != m_Labels.end() && iPosition == -1; x++, p++)

    // check, if input field was found
    if (iPosition != -1)
    {
        // check, if input widget is a QDoubleSpinBox
        if (dynamic_cast< ::QDoubleSpinBox* >(m_InputWidgets[iPosition]) != nullptr)
        {
            // check, if input field is visible
            if (static_cast< ::QDoubleSpinBox* >(m_InputWidgets[iPosition])->isVisible())
            {
                // set input field (double)
                static_cast< ::QDoubleSpinBox* >(m_InputWidgets[iPosition])->setValue(dValue);

                // press related input button to activate update
                m_InputButtons[iPosition]->click();

            } // if (static_cast< ::QDoubleSpinBox* >(m_InputWidgets[iPosition])->isVisible())
            else
            {
                // print warning
                ::std::cerr << "warning in CDMIQWidget::SetDoubleValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" is currently not visible" << ::std::endl;
                
            } // else

        } // if (dynamic_cast< ::QDoubleSpinBox* >(m_InputWidgets[iPosition]) != nullptr)
        else
        {
            // print warning
            ::std::cerr << "warning in CDMIQWidget::SetDoubleValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" is not a double field" << ::std::endl;
            ::std::cerr << typeid (*m_InputWidgets[p]).name() << ::std::endl;

        } // else

    } // if (iPosition != -1)
    else
    {
        // print warning
        ::std::cerr << "warning in CDMIQWidget::SetDoubleValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" not found" << ::std::endl;

    } // else
    
    // unlock mutex
    m_Mutex.unlock();
    
    #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "DEBUG: finished CDMIQWidget::SetDoubleValue()" << ::std::endl;
    #endif
  // Bouml preserved body end 000F3282

} // void CDMIQWidget::SetDoubleValue()  throw() 




void CDMIQWidget::SetIntegerValue(const QString & InputName, int iValue)   
{
  // Bouml preserved body begin 000F3302
    decltype (m_Labels.begin()) x;
    int                         p(0);
    int                         iPosition(-1);
    


    // lock mutex
    m_Mutex.lock();

    // try to find input field in labels
    for (x = m_Labels.begin() + m_OutputWidgets.size(), p = 0; x != m_Labels.end() && iPosition == -1; x++, p++)
    {
        // check, if current label corresponds to input field
        if ((*x)->text() == InputName)
        {
            // store position
            iPosition = p;
            
        } // if ((*x)->text() == InputName)
        
    } // for (x = m_Labels.begin() + m_OutputWidgets.size(), p = 0; x != m_Labels.end() && iPosition == -1; x++, p++)

    // check, if input field was found
    if (iPosition != -1)
    {
        // check, if input widget is a QSpinBox
        if (dynamic_cast< ::QSpinBox* >(m_InputWidgets[iPosition]) != nullptr)
        {
            // check, if input field is visible
            if (static_cast< ::QSpinBox* >(m_InputWidgets[iPosition])->isVisible())
            {
                // set input field (integer)
                static_cast< ::QSpinBox* >(m_InputWidgets[iPosition])->setValue(iValue);

                // press related input button to activate update
                m_InputButtons[iPosition]->click();
                
            } // if (static_cast< ::QSpinBox* >(m_InputWidgets[iPosition])->isVisible())
            else
            {
                // print warning
                ::std::cerr << "warning in CDMIQWidget::SetIntegerValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" is currently not visible" << ::std::endl;

            } // else

        } // if (dynamic_cast< ::QSpinBox* >(m_InputWidgets[iPosition]) != nullptr)
        else
        {
            // print warning
            ::std::cerr << "warning in CDMIQWidget::SetIntegerValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" is not a integer field" << ::std::endl;

        } // else
            
    } //  if (iPosition != -1)
    else
    {
        // print warning
        ::std::cerr << "warning in CDMIQWidget::SetIntegerValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" not found" << ::std::endl;
        
    } // else
    
    // unlock mutex
    m_Mutex.unlock();
  // Bouml preserved body end 000F3302

} // void CDMIQWidget::SetIntegerValue()  throw() 




void CDMIQWidget::SetStringValue(const QString & InputName, const QString & Value)   
{
  // Bouml preserved body begin 000F3382
    decltype (m_Labels.begin()) x;
    int                         p(0);
    int                         iPosition(-1);



    

    #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "DEBUG: started CDMIQWidget::SetStringValue()" << ::std::endl;
    #endif
    
    // lock mutex
    m_Mutex.lock();
    
    // try to find input field in labels
    for (x = m_Labels.begin() + m_OutputWidgets.size(), p = 0; x != m_Labels.end() && iPosition == -1; x++, p++)
    {
        // check, if current label corresponds to input field
        if ((*x)->text() == InputName)
        {
            // store position
            iPosition = p;
            
        } // if ((*x)->text() == InputName)
        
    } // for (x = m_Labels.begin() + m_OutputWidgets.size(), p = 0; x != m_Labels.end() && iPosition == -1; x++, p++)

    // check, if input field was found
    if (iPosition != -1)
    {
        // check, if input widget is a QLineEdit
        if (dynamic_cast< ::QLineEdit* >(m_InputWidgets[iPosition]) != nullptr)
        {
            // check, if input field is visible
            if (static_cast< ::QLineEdit* >(m_InputWidgets[iPosition])->isVisible())
            {
                // set input field (string)
                static_cast< ::QLineEdit* >(m_InputWidgets[iPosition])->setText(Value);

                // press related input button to activate update
                m_InputButtons[iPosition]->click();

            } // if (static_cast< ::QLineEdit* >(m_InputWidgets[iPosition])->isVisible())
            else
            {
                // print warning
                ::std::cerr << "warning in CDMIQWidget::SetStringValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" is currently not visible" << ::std::endl;

            } // else

        } // if (dynamic_cast< ::QLineEdit* >(m_InputWidgets[iPosition]) != nullptr)
        else
        {
            // print warning
            ::std::cerr << "warning in CDMIQWidget::SetStringValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" is not a string field" << ::std::endl;

        } // else 

    } // if (p != -1)
    else
    {
        // print warning
        ::std::cerr << "warning in CDMIQWidget::SetStringValue() in " << __FILE__ << ":" << __LINE__ << " -> input field \"" << InputName.toStdString() << "\" not found" << ::std::endl;
        
    } // else
    
    // unlock mutex
    m_Mutex.unlock();
    
    #ifdef __OETCS_DF_DMI_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "DEBUG: finished CDMIQWidget::SetStringValue()" << ::std::endl;
    #endif
  // Bouml preserved body end 000F3382

} // void CDMIQWidget::SetStringValue()  throw() 






} // namespace oETCS::DF

} // namespace oETCS
