/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief empty (no implemented methods) classes for the dbus adaptor classes
 */

#ifndef __OETCS_DF_PS_ADAPTORSTUBSMOC_H__
#define __OETCS_DF_PS_ADAPTORSTUBSMOC_H__




#include <QObject>
#include <QString>
#include <list>
#include <QBitArray>





namespace oETCS {

namespace DF {

namespace PS {


/*!
 *  \brief empty/stub class for the ::oETCS::DF::ABaliseDeviceIn adapter
 *
 *  This class is meant to be extened for any certain in Balise device implementation.
 */
class CBaliseDeviceIn : public ::QObject 
{
  Q_OBJECT
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] pParent optional pointer to a parent object, must be used,
     *                        if stub is exetened by inheritance
     */
    CBaliseDeviceIn(::QObject * const pParent = nullptr) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CBaliseDeviceIn() throw();
    
    


  Q_SIGNALS:
    /*!
     *  \brief signal for sending a new telegram
     *
     *  This signal should be emitted, if a new telegram was received by the HW.
     *
     *  \param[in] Bytes   array of bytes of the new telegram
     *  \param[in] iBits   number of bits only used from the beginning in the byte array
     */
    void NewTelegram(const QByteArray & Bytes, int iBits);
    
    

    /*!
     *  \brief signal for sending an error
     *
     *  This signal should be emitted in the case that an error in the HW ocurres.
     *
     *  \param[in] ErrorMessage textual description of the error
     */
    void Error(const ::QString& ErrorMessage);
    
    


  private:
    /*!
     *  \brief list of telegrams to be send (by polling/calling)
     */
    ::std::list< ::QBitArray > m_Telegrams;
    
    


}; // class CBaliseDeviceIn : public ::QObject



/*!
 *  \brief empty/stub class for the ::oETCS::DF::ABaliseDeviceOut adapter
 *
 *  This class is meant to be extened for any certain in Balise device implementation.
 */
class CBaliseDeviceOut : public ::QObject 
{
  Q_OBJECT
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] pParent optional pointer to a parent object, must be used,
     *                        if stub is exetened by inheritance
     */
    CBaliseDeviceOut(::QObject * const pParent = nullptr) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CBaliseDeviceOut() throw();
    
    

public Q_SLOTS:
    /*!
     *  \brief sends a telegram 
     *
     *  This method should be reimplemented or extened for a concrete HW device.
     *
     *  \param[in] Bits telegram to send as bits
     */
    virtual void SendTelegram(const ::QBitArray& Bits);
    
    


  Q_SIGNALS:
    /*!
     *  \brief signal for sending an error
     *
     *  This signal should be emitted in the case that an error in the HW ocurres.
     *
     *  \param[in] ErrorMessage textual description of the error
     */
    void Error(const ::QString& ErrorMessage);
    
    


}; // class CBaliseDeviceOut : public ::QObject



/*!
 *  \brief empty/stub class for the ::oETCS::DF::AEmergencyBrake adapter
 *
 *  This class is meant to be extened for any certain in emergency brake device implementation.
 */
class CEmergencyBrake : public ::QObject 
{
  Q_OBJECT
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] pParent optional pointer to a parent object, must be used,
     *                        if stub is exetened by inheritance
     */
    CEmergencyBrake(::QObject * const pParent = nullptr) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CEmergencyBrake() throw();
    
    

public Q_SLOTS:
    /*!
     *  \brief gets the activation state
     *
     *  This method should be reimplemented or extened for a concrete HW device.
     *
     *  \return activation flag of the emergency brake
     */
    virtual bool GetActivation();
    
    

    /*!
     *  \brief sets the activation state
     *
     *  This method should be reimplemented or extened for a concrete HW device.
     *
     *  \param[in] bActivated the new activation flag
     */
    virtual void SetActivation(bool bActivated);
    
    


  Q_SIGNALS:
    /*!
     *  \brief signal for sending an error
     *
     *  This signal should be emitted in the case that an error in the HW ocurres.
     *
     *  \param[in] ErrorMessage textual description of the error
     */
    void Error(const ::QString& ErrorMessage);
    
    


}; // class CEmergencyBrake : public ::QObject



/*!
 *  \brief empty/stub class for the ::oETCS::DF::AServiceBrake adapter
 *
 *  This class is meant to be extened for any certain in service brake device implementation.
 */
class CServiceBrake : public ::QObject 
{
  Q_OBJECT
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] pParent optional pointer to a parent object, must be used,
     *                        if stub is exetened by inheritance
     */
    CServiceBrake(::QObject * const pParent = nullptr) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CServiceBrake() throw();
    
    

public Q_SLOTS:
    /*!
     *  \brief gets the activation intensity
     *
     *  This method should be reimplemented or extened for a concrete HW device.
     *
     *  \return activation intensity of the service brake in [%]
     */
    virtual double GetIntensity();
    
    

    /*!
     *  \brief sets the activation intensity
     *
     *  This method should be reimplemented or extened for a concrete HW device.
     *
     *  \param[in] bIntensity the new activation intensity in [%]
     */
    virtual void SetIntensity(const double& dIntensity);
    
    


  Q_SIGNALS:
    /*!
     *  \brief signal for sending an error
     *
     *  This signal should be emitted in the case that an error in the HW ocurres.
     *
     *  \param[in] ErrorMessage textual description of the error
     */
    void Error(const ::QString& ErrorMessage);
    
    


}; // class CServiceBrake : public ::QObject



/*!
 *  \brief empty/stub class for the ::oETCS::DF::AOdometer adapter
 *
 *  This class is meant to be extened for any certain in odometer device implementation.
 */
class COdometer : public ::QObject 
{
  Q_OBJECT
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] pParent optional pointer to a parent object, must be used,
     *                        if stub is exetened by inheritance
     */
    COdometer(::QObject * const pParent = nullptr) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~COdometer() throw();
    
    

public Q_SLOTS:
    /*!
     *  \brief gets the current absolute position
     *
     *  This method should be reimplemented or extened for a concrete HW device.
     *
     *  \return current absolute position in [m]
     */
    virtual double GetAbsolutePosition();
    
    

    /*!
     *  \brief gets the current velocity
     *
     *  This method should be reimplemented or extened for a concrete HW device.
     *
     *  \return current velocity in [km/h]
     */
    virtual double GetVelocity();
    
    


  Q_SIGNALS:
    /*!
     *  \brief signal for sending an error
     *
     *  This signal should be emitted in the case that an error in the HW ocurres.
     *
     *  \param[in] ErrorMessage textual description of the error
     */
    void Error(const ::QString& ErrorMessage);
    
    


}; // class COdometer : public ::QObject





} // namespace oETCS::DF::PS

} // namespace oETCS::DF

} // namespace oETCS

#endif // __ OETCS_DF_PS_ADAPTORSTUBSMOC_H__
