/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief C wrapper functions for the simulative PSM classes
 */

#ifndef __OETCS_DF_PS_SIM_WRAPPERFUNCTIONS_H__
#define __OETCS_DF_PS_SIM_WRAPPERFUNCTIONS_H__


#include <QtCore>
#include <QtDBus>
#include <QMutex>
#include "oETCS/DF/OdometerAdaptor.h"
#include "oETCS/DF/EmergencyBrakeAdaptor.h"
#include "oETCS/DF/ServiceBrakeAdaptor.h"
#include "oETCS/DF/BaliseDeviceInAdaptor.h"
#include "oETCS/DF/BaliseDeviceOutAdaptor.h"
#include "oETCS/DF/PS/SIM/DMIInterface.h"
#include "Simulation.h"





/*  public:
    int dummy;*/
    
    

#define __EVC_DBUS__ "tcp:host=192.168.56.1,port=2222"



extern "C"
{

    /*!
     *  \brief global telegram for ETCS Application Level switchtes from 0 to 1
     */
    extern const int            G_TELEGRAM_LEVEL_0_TO_1[];



    /*!
     *  \brief global size of global telegram for ETCS Application Level switchtes from 0 to 1
     */
    extern const int            G_SIZE_TELEGRAM_LEVEL_0_TO_1;



    /*!
     *  \brief global telegram for ETCS Application Level switchtes from 0 to 1
     */
    extern const int            G_TELEGRAM_LEVEL_STOP_IN_SR[];



    /*!
     *  \brief global size of global telegram for ETCS Application Level switchtes from 0 to 1
     */
    extern const int            G_SIZE_TELEGRAM_STOP_IN_SR;



    /*!
     *  \brief thread function for the QCoreApplication instance
     *
     *  \param[out] pInitialised pointer to boolean to the set initialisation status
     */
    static void QAppThread(bool * pInitialised);
    
    

    /*!
     *	\brief PSM D-Bus adaptor initialisations
     */
    void InitPSM();
    
    

    /*!
     *  \brief initialises the module global DMI proxy
     * 
     *  \return !=0, if the DMI proxy was initialised sucessfully
     */
    int InitDMI();
    
    

    /*!
     * 	\brief gets the DMI input status
     *
     *	\return !=0, if a any (new) input field was found
     */
    int DMIHasInput();
    
    

    /*!
     *  \brief gets a certain DMI output value
     * 
     *  \param[in]  pName   pointer to name of the output field
     *  \param[out] ppValue pointer of pointer to value of the output field, de- and allocated by this function
     */
    void GetDMIOutput(const char * const pName, char ** const ppValue);
    
    

    /*!
     *	\brief sets a boolean input value in the DMI
     * 
     * 	\param[in] pInputName 	literal name of the input field
     * 	\param[in] iValue 	input value
     */
    void SetDMIBooleanValue(const char * const pInputName, int iValue);
    
    

    /*!
     *	\brief sets an integer input value in the DMI
     * 
     * 	\param[in] pInputName 	literal name of the input field
     * 	\param[in] iValue 	input value
     */
    void SetDMIIntegerValue(const char * const pInputName, int iValue);
    
    

    /*!
     *	\brief sets a double input value in the DMI
     * 
     * 	\param[in] pInputName 	literal name of the input field
     * 	\param[in] iValue 	input value
     */
    void SetDMIDoubleValue(const char * const pInputName, double dValue);
    
    

    /*!
     *	\brief sets a string input value in the DMI
     * 
     * 	\param[in] pInputName 	literal name of the input field
     * 	\param[in] pValue 	input value (C-string)
     */
    void SetDMIStringValue(const char * const pInputName, const char * const pValue);
    
    

    /*!
     *  \brief sets a telegram as received for the PIM
     *
     *  \param[in] pTelegram pointer to bytes of the telegram
     *  \param[in] iBits number bits included in the telegram
     */
    void SetNewInputTelegram(char * pTelegram, int iBits);
    
    

    /*!
     *  \brief sets a telegram as received for the PIM in a binary format
     *
     *  \param[in] pTelegram pointer to bits of the telegram
     *  \param[in] iBits number bits included in the telegram
     */
    void SetNewInputTelegramBinary(const int * const pTelegram, int iBits);
    
    

    /*!
     * 	\brief gets the type of the first DMI input
     * 
     * 	\param[out] pDMIType 	pointer to the integer with the current DMI type value
     */
    void EvaluateDMIInputs(int * pDMIType);
    
    

    /*!
     *  \brief gets the current speed from the odometer object
     *
     *  \return current speed [km/h]
     */
    double GetVelocity();
    
    

    /*!
     *  \brief sets the speed in the odomoeter object
     *
     *  \param[in] dVelocity new speed in [km/h]
     */
    void SetVelocity(double dVelocity);
    
    

    /*!
     *  \brief (re)sets the position of the odometer object
     *
     *  \param[in] dPosition new position in [m]
     */
    void ResetPosition(double dPosition);
    
    

    /*!
     *  \brief gets the current position from the odometer object
     *
     *  \return current position [m]
     */
    double GetPosition();
    
    

    /*!
     *  \brief sets the activation intensity
     *
     *  \param[in] bIntensity the new activation intensity in [%]
     */
    void SetServiceIntensity(double dIntensity);
    
    

    /*!
     *  \brief gets the activation intensity
     *
     *  \return activation intensity of the service brake in [%]
     */
    double GetServiceIntensity();
    
    

    /*!
     *  \brief sets the virtual mass
     *
     *  \param[in] dMass new mass in [kg]
     */
    void SetMass(double dMass);
    
    

    /*!
     *  \brief sets the virtual adhesion
     *
     *  \param[in] dAdhesion new virtual adhesion factor (0...1)
     */
    void SetAdhesion(double dAdhesion);
    
    

    /*!
     *  \brief sets the activation state
     *
     *  \param[in] iActivated the new activation flag (!=0: true)
     */
    void SetEmergencyActivation(int iActication);
    
    

    /*!
     *  \brief gets the activation state
     *
     *  \return activation flag of the emergency brake (!=0: true)
     */
    int GetEmergencyActivation();
    
    

} // extern "C"





#endif // __ OETCS_DF_PS_SIM_WRAPPERFUNCTIONS_H__
