/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \file
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief main file for generator binary
 *
 *  File holding the main() function and CMain class for the openETCS
 *  generator binary execution.
 */

#ifndef __OETCS_GEN_MAIN_H__
#define __OETCS_GEN_MAIN_H__


#include <unistd.h>

#include <string>
#include "../../DSM/GOPPRRTransformer.h"
#include "CPPGenerator.h"
#include "VMgenerator.h"
#include "BuildGenerator.h"
#include "../../GOPPRR/ConstraintChecker.h"
#include <sstream>



namespace oETCS {

namespace GEN {


/*!
 *  \brief generator main class
 *  
 *  Main class holding routines for command line binary.
 *  Typically invoked by main() function. Combines functionality
 *  of all generator, GOPPRR and DSM components.
 */
class CMain 
{
  public:
    /*!
     *  \brief general constructor
     *
     *  general constructor
     '
     '  \param[in] ppArguements argument vector with char pointers
     *  \param[in] iNumberArguments number of arguments in vector
     */
    CMain(char ** ppArguements = 0, int iNumberArguments = 0) throw();
    
    

    /*!
     *  \brief destructor
     *
     *  destructor
     */
    virtual ~CMain() throw();
    
    

    /*!
     *  \brief executes the main functionality of the generator
     *
     *  \return the execution code
     */
    int Execute() throw();
    
    


  private:
    /*!
     *  \brief internal helper method
     *
     *  Prints the usage of the command line arguments.
     */
    void PrintUsage() throw();
    
    

    /*!
     *  \brief flag for printing the usage
     */
    bool m_bHelp;
    
    

    /*!
     *  \brief flag for genrating source
     */
    bool m_bSource;
    
    

    /*!
     *  \brief flag for genrating build config
     */
    bool m_bBuildConfig;
    
    

    /*!
     *  \brief flag for genrating build config
     */
    bool m_bVMConfig;
    
    

    /*!
     *  \brief flag for applying model checking
     */
    bool m_bContraintChecking;
    
    

    /*!
     *  \brief literal name of the binary
     */
    ::std::string m_BinaryName;
    
    

    /*!
     *  \brief file name of the XML model file
     */
    ::std::string m_XMLFile;
    
    

    /*!
     *  \brief constraint file name
     */
    ::std::string m_ContraintFile;
    
    

    /*!
     *  \brief output directory for generate files
     */
    ::std::string m_OutputDirectory;
    
    

    /*!
     *  \brief lexer object
     *
     *  Lexer object for creating syntax trees.
     */
    ::DSM::CGOPPRRTransformer m_Lexer;
    
    

    /*!
     *  \brief C++ Generator
     *
     *  C++ generator object.
     */
    oETCS::GEN::CCPPGenerator m_CppGenerator;
    
    

    /*!
     *  \brief VM geneator
     *
     *  Generator object for configuration files of virtual machines.
     */
    oETCS::GEN::CVMGenerator m_VMGenerator;
    
    

    /*!
     *  \brief build Generator
     *
     *  Build generator object.
     */
    oETCS::GEN::CBuildGenerator m_BuildGenerator;
    
    

    /*!
     *  \brief contraint checker object
     */
    GOPPRR::CConstraintChecker m_ConstraintChecker;
    
    

    /*!
     *  \brief error message stream
     */
    ::std::stringstream m_ErrorMessage;
    
    


}; // class CMain





} // namespace oETCS::GEN

} // namespace oETCS

#endif // __ OETCS_GEN_MAIN_H__
