/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief unit testing module
 */

#include "UnitTesting.h"
#include "../DF/FunctionBlocks.h"
#include "../DF/Condition.h"
#include "../DF/Storage.h"
#include "../DF/ControlFlow.h"
#include "../../DSM/GOPPRRTransformer.h"
#include "../../GOPPRR/ConstraintChecker.h"
#include "../GEN/CPPGenerator.h"


// register all test suites
CPPUNIT_TEST_SUITE_REGISTRATION( ::oETCS::UT::CFunctionBlock );
CPPUNIT_TEST_SUITE_REGISTRATION( ::oETCS::UT::CLanguage );
CPPUNIT_TEST_SUITE_REGISTRATION( ::oETCS::UT::CEVCStateMachine );
CPPUNIT_TEST_SUITE_REGISTRATION( ::oETCS::UT::CControlFlow );
CPPUNIT_TEST_SUITE_REGISTRATION( ::oETCS::UT::CGOPPRRTransformer );
CPPUNIT_TEST_SUITE_REGISTRATION( ::oETCS::UT::CGenerator );

namespace oETCS {

namespace UT {


CFunctionBlock::CFunctionBlock()  throw()  
:m_pAnd(0),
 m_pOr(0),
 m_pEVCCondition(0),
 m_pCondition(0),
 m_pLevelCondition(0),
 m_pComBlockIn(0),
 m_pComBlockOut(0),
 m_pXor(0),
 m_pNot(0),
 m_pSum(0),
 m_pSubstraction(0),
 m_pDivision(0),
 m_pMultiplication(0),
 m_pDoubleEqual(0),
 m_pIntEqual(0),
 m_pStringEqual(0),
 m_pDoubleEqualOrGreater(0),
 m_pDoubleGreater(0),
 m_pIntGreater(0),
 m_pDoubleArrayAccessor(0),
 m_pBoolGate(0),
 m_pBrakingToTargetSpeed(0),
 m_pCeelingSpeedControl(0),
 m_pStringGate(0),
 m_pDoubleGate(0),
 m_pBoolSwitch(0),
 m_pDoubleSwitch(0),
 m_pStringSwitch(0),
 m_pEnteredTrigger(0),
 m_pStorage(0),
 m_pBitStorage(0),
 m_bBoolInput(false),
 m_bBoolInput2(false),
 m_bBoolInput3(false),
 m_bBoolInput4(false),
 m_dDoubleInput(0),
 m_DoubleArrayInput(::std::vector< double >(0)),
 m_iIntInput(0),
 m_StringInput("")
{
  // Bouml preserved body begin 0007B802
    // empty constructor
  // Bouml preserved body end 0007B802

} //  CFunctionBlock::CFunctionBlock()  throw() 




void CFunctionBlock::setUp()  throw(::std::bad_alloc)  
{
  // Bouml preserved body begin 0007B882
    ::oETCS::DF::CVariable*                         pVariable(0);
    ::oETCS::DF::CVariable*                         pHeader(0);
    ::oETCS::DF::CPacket*                           pPacket(0);
    ::oETCS::DF::CTelegram*                         pTelegram(0);
    ::oETCS::DF::CBaliseDeviceIn*                   pBaliseIn(0);
    ::oETCS::DF::CBaliseDeviceOut*                  pBaliseOut(0);
    ::oETCS::DF::CFlow< bool >*                     pBoolFlow(0);
    ::oETCS::DF::CFlow< double >*                   pDoubleFlow(0);
    ::oETCS::DF::CFlow< ::std::vector< double > >*  pDoubleArrayFlow(0);
    ::oETCS::DF::CFlow< signed int >*               pIntFlow(0);
    ::oETCS::DF::CFlow< ::std::string >*            pStringFlow(0);
    
    
    
    // create storage object, flows and language objects first, because of dependencies
    m_pStorage              = new ::oETCS::DF::CStorage(&m_StateMachine);
    pBoolFlow               = new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine);
    pDoubleFlow             = new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine);
    pDoubleArrayFlow        = new ::oETCS::DF::CFlow< ::std::vector< double> >(&m_DoubleArrayInput, &m_StateMachine);
    pIntFlow                = new ::oETCS::DF::CFlow< signed int >(&m_iIntInput, &m_StateMachine);
    pStringFlow             = new ::oETCS::DF::CFlow< ::std::string >(&m_StringInput, &m_StateMachine);
    m_pStorage->m_BoolValue.push_back(pBoolFlow);
    m_pStorage->m_DoubleValue.push_back(pDoubleFlow);
    m_pStorage->m_DoubleArrayValue.push_back(pDoubleArrayFlow);
    m_pStorage->m_IntValue.push_back(pIntFlow);
    m_pStorage->m_StringValue.push_back(pStringFlow);

    m_pBitStorage            = new ::oETCS::DF::CStorage(&m_StateMachine);
    m_pStorage->m_BitValue.push_back(new ::oETCS::DF::CBitFlow(m_pBitStorage, &m_StateMachine));
    pVariable               = new ::oETCS::DF::CVariable(&m_StateMachine, 1, ::oETCS::DF::CVariable::DOUBLE, 32);
    pVariable->m_BitValues.push_back(new ::oETCS::DF::CBitFlow(m_pStorage, &m_StateMachine));
    pHeader                 = new ::oETCS::DF::CVariable(&m_StateMachine, 2, ::oETCS::DF::CVariable::INTEGER, 16);
    pPacket                 = new ::oETCS::DF::CPacket(&m_StateMachine, 5, ::std::vector< ::oETCS::DF::CVariable* >(1, pVariable));
    pTelegram               = new ::oETCS::DF::CTelegram(&m_StateMachine,::std::vector< ::oETCS::DF::CVariable* >(1, pHeader), ::std::vector< ::oETCS::DF::CPacket* >(1, pPacket));
    pBaliseIn               = new ::oETCS::DF::CBaliseDeviceIn(::std::vector< ::oETCS::DF::CTelegram* >(1, pTelegram));
    pBaliseOut              = new ::oETCS::DF::CBaliseDeviceOut(::std::vector< ::oETCS::DF::CTelegram* >(1, pTelegram));
    
    
    // create all high level function block test objects and their flows
    m_pAnd                  = new ::oETCS::DF::CAnd(&m_StateMachine);
    m_pAnd->m_Result.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pOr                   = new ::oETCS::DF::COr(&m_StateMachine);
    m_pOr->m_Result.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pEVCCondition         = new ::oETCS::DF::CEVCCondition(&m_StateMachine);
    m_pCondition            = new ::oETCS::DF::CCondition(&m_StateMachine);
    m_pLevelCondition       = new ::oETCS::DF::CLevelCondition(&m_StateMachine, "TEST");
    m_pComBlockIn           = new ::oETCS::DF::CComBlockIn(&m_StateMachine, ::std::vector< ::oETCS::DF::CBaliseDeviceIn* >(1, pBaliseIn));
    m_pComBlockOut          = new ::oETCS::DF::CComBlockOut(&m_StateMachine, ::std::vector< ::oETCS::DF::CBaliseDeviceOut* >(1, pBaliseOut));
    m_pXor                  = new ::oETCS::DF::CXor(&m_StateMachine);
    m_pXor->m_Result.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pNot                  = new ::oETCS::DF::CNot(&m_StateMachine);
    m_pNot->m_Result.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pSum                  = new ::oETCS::DF::CSum(&m_StateMachine);
    m_pSum->m_Sum.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pSubstraction         = new ::oETCS::DF::CSubstraction(&m_StateMachine);
    m_pSubstraction->m_Difference.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pDivision             = new ::oETCS::DF::CDivision(&m_StateMachine);
    m_pDivision->m_Quotient.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pMultiplication       = new ::oETCS::DF::CMultiplication(&m_StateMachine);
    m_pMultiplication->m_Product.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pDoubleEqual          = new ::oETCS::DF::CDoubleEqual(&m_StateMachine);
    m_pDoubleEqual->m_Equal.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pIntEqual             = new ::oETCS::DF::CIntEqual(&m_StateMachine);
    m_pIntEqual->m_Equal.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pStringEqual          = new ::oETCS::DF::CStringEqual(&m_StateMachine);
    m_pStringEqual->m_Equal.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pDoubleEqualOrGreater = new ::oETCS::DF::CDoubleEqualOrGreater(&m_StateMachine);
    m_pDoubleEqualOrGreater->m_EqualOrGreater.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pDoubleGreater        = new ::oETCS::DF::CDoubleGreater(&m_StateMachine);
    m_pDoubleGreater->m_Greater.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pIntGreater           = new ::oETCS::DF::CIntGreater(&m_StateMachine);
    m_pIntGreater->m_Greater.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pDoubleArrayAccessor  = new ::oETCS::DF::CDoubleArrayAccessor(&m_StateMachine);
    m_pDoubleArrayAccessor->m_Value.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pBoolGate             = new ::oETCS::DF::CBoolGate(&m_StateMachine);
    m_pBoolGate->m_GateOutput.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pBrakingToTargetSpeed = new ::oETCS::DF::CBrakingToTargetSpeed(&m_StateMachine);
    m_pBrakingToTargetSpeed->m_EmergencyBrakeValue.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pBrakingToTargetSpeed->m_ServiceBrakeValue.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pBrakingToTargetSpeed->m_EmergencyBrakeApplied.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput2, &m_StateMachine));
    m_pBrakingToTargetSpeed->m_ServiceBrakeApplied.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput3, &m_StateMachine));
    m_pBrakingToTargetSpeed->m_SpeedWarning.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput4, &m_StateMachine));
    m_pBrakingToTargetSpeed->m_ServiceBrakeValue.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pCeelingSpeedControl  = new ::oETCS::DF::CCeelingSpeedControl(&m_StateMachine); 
    m_pCeelingSpeedControl->m_EmergencyBrakeValue.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pCeelingSpeedControl->m_EmergencyBrakeApplied.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput2, &m_StateMachine));
    m_pCeelingSpeedControl->m_ServiceBrakeApplied.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput3, &m_StateMachine));
    m_pCeelingSpeedControl->m_SpeedWarning.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput4, &m_StateMachine));
    m_pCeelingSpeedControl->m_ServiceBrakeValue.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pStringGate           = new ::oETCS::DF::CStringGate(&m_StateMachine);
    m_pStringGate->m_GateOutput.push_back(new ::oETCS::DF::CFlow< ::std::string >(&m_StringInput, &m_StateMachine));
    m_pDoubleGate           = new ::oETCS::DF::CDoubleGate(&m_StateMachine);
    m_pDoubleGate->m_GateOutput.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pBoolSwitch           = new ::oETCS::DF::CBoolSwitch(&m_StateMachine);
    m_pBoolSwitch->m_SwitchOutput.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
    m_pDoubleSwitch         = new ::oETCS::DF::CDoubleSwitch(&m_StateMachine);
    m_pDoubleSwitch->m_SwitchOutput.push_back(new ::oETCS::DF::CFlow< double >(&m_dDoubleInput, &m_StateMachine));
    m_pStringSwitch         = new ::oETCS::DF::CStringSwitch(&m_StateMachine);
    m_pStringSwitch->m_SwitchOutput.push_back(new ::oETCS::DF::CFlow< ::std::string >(&m_StringInput, &m_StateMachine));
    m_pEnteredTrigger       = new ::oETCS::DF::CEnteredTrigger(&m_StateMachine);
    m_pEnteredTrigger->m_Entered.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, &m_StateMachine));
  // Bouml preserved body end 0007B882

} // void CFunctionBlock::setUp()  throw(::std::bad_alloc) 




void CFunctionBlock::tearDown()  throw()  
{
  // Bouml preserved body begin 0007B902
    // all function block objects are deleted by destructor of m_StateMachine
  // Bouml preserved body end 0007B902

} // void CFunctionBlock::tearDown()  throw() 




void CFunctionBlock::TestAnd()  throw(::std::exception)  
{
  // Bouml preserved body begin 0007EC82
    // reset boolean input of fixture
    m_bBoolInput = false;
            
    // set inputs of CAnd object
    m_pAnd->m_bInput1 = true;
    m_pAnd->m_bInput2 = true;
    
    // calculate result
    m_pAnd->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean and operation failed", m_bBoolInput() == true);
    
    // reset inputs of CAnd object
    m_pAnd->m_bInput1 = false;
    m_pAnd->m_bInput2 = true;

    // calculate result
    m_pAnd->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean and operation failed", m_bBoolInput() == false);
    
    // reset boolean input of fixture
    m_bBoolInput = true;

    // set inputs of CAnd object
    m_pAnd->m_bInput1 = true;
    m_pAnd->m_bInput2 = false;

    // calculate result
    m_pAnd->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean and operation failed", m_bBoolInput() == false);
    
    // reset boolean input of fixture
    m_bBoolInput = true;

    // set inputs of CAnd object
    m_pAnd->m_bInput1 = false;
    m_pAnd->m_bInput2 = false;

    // calculate result
    m_pAnd->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean and operation failed", m_bBoolInput() == false);
  // Bouml preserved body end 0007EC82

} // void CFunctionBlock::TestAnd()  throw(::std::exception) 




void CFunctionBlock::TestOr()  throw(::std::exception)  
{
  // Bouml preserved body begin 0007ED02
    // reset boolean input of fixture
    m_bBoolInput = false;
            
    // set inputs of COr object
    m_pOr->m_bInput1 = true;
    m_pOr->m_bInput2 = true;
    
    // calculate result
    m_pOr->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean or operation failed", m_bBoolInput() == true);
    
    // reset boolean input of fixture
    m_bBoolInput = false;
    
    // reset inputs of COr object
    m_pOr->m_bInput1 = false;
    m_pOr->m_bInput2 = true;

    // calculate result
    m_pOr->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean or operation failed", m_bBoolInput() == true);
    
    // reset boolean input of fixture
    m_bBoolInput = false;

    // set inputs of COr object
    m_pOr->m_bInput1 = true;
    m_pOr->m_bInput2 = false;

    // calculate result
    m_pOr->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean or operation failed", m_bBoolInput() == true);
    
    // reset boolean input of fixture
    m_bBoolInput = true;

    // set inputs of COr object
    m_pOr->m_bInput1 = false;
    m_pOr->m_bInput2 = false;

    // calculate result
    m_pOr->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean or operation failed", m_bBoolInput() == false);
  // Bouml preserved body end 0007ED02

} // void CFunctionBlock::TestOr()  throw(::std::exception) 




void CFunctionBlock::TestXor()  throw(::std::exception)  
{
  // Bouml preserved body begin 0007ED82
    // reset boolean input of fixture
    m_bBoolInput = true;
            
    // set inputs of CXor object
    m_pXor->m_bInput1 = true;
    m_pXor->m_bInput2 = true;
    
    // calculate result
    m_pXor->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean exclusive or operation failed", m_bBoolInput() == false);
    
    // reset boolean input of fixture
    m_bBoolInput = false;
    
    // reset inputs of CXor object
    m_pXor->m_bInput1 = false;
    m_pXor->m_bInput2 = true;

    // calculate result
    m_pXor->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean exclusive or operation failed", m_bBoolInput() == true);
    
    // reset boolean input of fixture
    m_bBoolInput = false;

    // set inputs of CXor object
    m_pXor->m_bInput1 = true;
    m_pXor->m_bInput2 = false;

    // calculate result
    m_pXor->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean exclusive or operation failed", m_bBoolInput() == true);
    
    // reset boolean input of fixture
    m_bBoolInput = true;

    // set inputs of CXor object
    m_pXor->m_bInput1 = false;
    m_pXor->m_bInput2 = false;

    // calculate result
    m_pXor->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean exclusive or operation failed", m_bBoolInput() == false);
  // Bouml preserved body end 0007ED82

} // void CFunctionBlock::TestXor()  throw(::std::exception) 




void CFunctionBlock::TestNot()  throw(::std::exception)  
{
  // Bouml preserved body begin 00080702
    // reset boolean input of fixture
    m_bBoolInput = true;
            
    // set inputs of CNot object
    m_pNot->m_bInput = true;
    
    // calculate result
    m_pNot->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean not operation failed", m_bBoolInput() == false);
    
    // reset boolean input of fixture
    m_bBoolInput = false;
    
    // set inputs of CNot object
    m_pNot->m_bInput = false;

    // calculate result
    m_pNot->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("boolean not operation failed", m_bBoolInput() == true);
  // Bouml preserved body end 00080702

} // void CFunctionBlock::TestNot()  throw(::std::exception) 




void CFunctionBlock::TestSum()  throw(::std::exception)  
{
  // Bouml preserved body begin 00080782
    const double        ADDEND1(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        ADDEND2(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        ADDEND3(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    
    
    
    // set inputs of CSum
    m_pSum->m_dAddend1 = ADDEND1;
    m_pSum->m_dAddend2 = ADDEND2;
    m_pSum->m_dAddend3 = ADDEND3;
    
    // calculate the result
    m_pSum->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("addition of three double values failed", m_dDoubleInput() == (ADDEND1 + ADDEND2 + ADDEND3));
  // Bouml preserved body end 00080782

} // void CFunctionBlock::TestSum()  throw(::std::exception) 




void CFunctionBlock::TestSubstraction()  throw(::std::exception)  
{
  // Bouml preserved body begin 00080802
    const double        MINUEND(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        SUBTRAHEND(200.0 * double(rand()) / double(RAND_MAX) - 100.0);

    
    
    // set inputs of CSubstraction
    m_pSubstraction->m_dMinuend         = MINUEND;
    m_pSubstraction->m_dSubtrahend      = SUBTRAHEND;
    
    // calculate the result
    m_pSubstraction->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("subtraction of double values failed", m_dDoubleInput() == (MINUEND - SUBTRAHEND));
  // Bouml preserved body end 00080802

} // void CFunctionBlock::TestSubstraction()  throw(::std::exception) 




void CFunctionBlock::TestDivision()  throw(::std::exception)  
{
  // Bouml preserved body begin 00080882
    const double        DIVIDEND(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        DIVISOR(200.0 * double(rand()) / double(RAND_MAX) + 0.1);
    const double        RESULT(DIVIDEND / DIVISOR);
    
    
    
    // set inputs of CDivision
    m_pDivision->m_dDividend    = DIVIDEND;
    m_pDivision->m_dDivisor     = DIVISOR;
    
    // calculate the result
    m_pDivision->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("division of double values failed", m_dDoubleInput() == RESULT);
  // Bouml preserved body end 00080882

} // void CFunctionBlock::TestDivision()  throw(::std::exception) 




void CFunctionBlock::TestMultiplication()  throw(::std::exception)  
{
  // Bouml preserved body begin 00080902
    const double        MULTIPLICATOR1(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        MULTIPLICATOR2(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        MULTIPLICATOR3(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        RESULT(MULTIPLICATOR1 * MULTIPLICATOR2 * MULTIPLICATOR3);
    
    
    
    // set inputs of CMultiplication
    m_pMultiplication->m_dMultiplicator1    = MULTIPLICATOR1;
    m_pMultiplication->m_dMultiplicator2    = MULTIPLICATOR2;
    m_pMultiplication->m_dMultiplicator3    = MULTIPLICATOR3;
    
    // calculate the result
    m_pMultiplication->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("multiplication of three double factors failed", abs(m_dDoubleInput() - RESULT) < 0.00001 );
  // Bouml preserved body end 00080902

} // void CFunctionBlock::TestMultiplication()  throw(::std::exception) 




void CFunctionBlock::TestDoubleEqual()  throw(::std::exception)  
{
  // Bouml preserved body begin 00080982
    const double        VALUE1(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        VALUE2(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    
    
    
    // set inputs of CDoubleEqual
    m_pDoubleEqual->m_dInput1 = VALUE1;
    m_pDoubleEqual->m_dInput2 = VALUE2;
    
    // calculate the result
    m_pDoubleEqual->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("double equal computation failed", m_bBoolInput() == false);
    
    // set inputs of CDoubleEqual
    m_pDoubleEqual->m_dInput1 = VALUE1;
    m_pDoubleEqual->m_dInput2 = VALUE1;

    // calculate the result
    m_pDoubleEqual->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("double equal computation failed", m_bBoolInput() == true);
  // Bouml preserved body end 00080982

} // void CFunctionBlock::TestDoubleEqual()  throw(::std::exception) 




void CFunctionBlock::TestIntEqual()  throw(::std::exception)  
{
  // Bouml preserved body begin 00080A02
    const int       VALUE1(rand());
    const int       VALUE2(rand());
    
    
    
    // set inputs of CIntEqual
    m_pIntEqual->m_iInput1 = VALUE1;
    m_pIntEqual->m_iInput2 = VALUE2;
    
    // calculate the result
    m_pIntEqual->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("integer equal computation failed", m_bBoolInput() == false);
    
    // set inputs of CDoubleEqual
    m_pIntEqual->m_iInput1 = VALUE1;
    m_pIntEqual->m_iInput2 = VALUE1;

    // calculate the result
    m_pIntEqual->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("integer equal computation failed", m_bBoolInput() == true);
  // Bouml preserved body end 00080A02

} // void CFunctionBlock::TestIntEqual()  throw(::std::exception) 




void CFunctionBlock::TestStringEqual()  throw(::std::exception)  
{
  // Bouml preserved body begin 00080A82
    unsigned int        x(0);
    ::std::string       Value1;
    ::std::string       Value2;
    

    
    // create random string
    for (x = 0; x < 20; x++)
    {
        // add random char to strings
        Value1 += char(double(rand()) / double(RAND_MAX) * 94.0 + 32.0);
        Value2 += char(double(rand()) / double(RAND_MAX) * 94.0 + 32.0);
        
    } // for (x = 0; x < 20; x++)
    
    // reset boolean input
    m_bBoolInput = true;
    
    // set input of CStringEqual
    m_pStringEqual->m_Input1 = Value1;
    m_pStringEqual->m_Input2 = Value2;
    
    // calculate the result
    m_pStringEqual->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("string equal computation failed", m_bBoolInput() == false);
    
    // reset boolean input
    m_bBoolInput = false;

    // set input of CStringEqual
    m_pStringEqual->m_Input1 = Value1;
    m_pStringEqual->m_Input2 = Value1;

    // calculate the result
    m_pStringEqual->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("string equal computation failed", m_bBoolInput() == true);
  // Bouml preserved body end 00080A82

} // void CFunctionBlock::TestStringEqual()  throw(::std::exception) 




void CFunctionBlock::TestDoubleEqualOrGreater()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082402
    const double        VALUE1(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        VALUE2(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    
    
    
    // set inputs of CDoubleEqualOrGreater
    m_pDoubleEqualOrGreater->m_dInput1 = VALUE1;
    m_pDoubleEqualOrGreater->m_dInput2 = VALUE2;
    
    // calculate the result
    m_pDoubleEqualOrGreater->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("double equal or greater (>=) computation failed", m_bBoolInput() == (VALUE1 >= VALUE2));
    
    // set inputs of CDoubleEqualOrGreater
    m_pDoubleEqualOrGreater->m_dInput1 = VALUE2;
    m_pDoubleEqualOrGreater->m_dInput2 = VALUE1;

    // calculate the result
    m_pDoubleEqualOrGreater->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("double equal or greater (>=) computation failed", m_bBoolInput() == (VALUE1 <= VALUE2));
    
    // set inputs of CDoubleEqualOrGreater
    m_pDoubleEqualOrGreater->m_dInput1 = VALUE1;
    m_pDoubleEqualOrGreater->m_dInput2 = VALUE1;

    // calculate the result
    m_pDoubleEqualOrGreater->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("double equal or greater (>=) computation failed", m_bBoolInput() == true);
  // Bouml preserved body end 00082402

} // void CFunctionBlock::TestDoubleEqualOrGreater()  throw(::std::exception) 




void CFunctionBlock::TestDoubleGreater()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082482
    const double        VALUE1(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double        VALUE2(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    
    
    
    // set inputs of CDoublGreater
    m_pDoubleGreater->m_dInput1 = VALUE1;
    m_pDoubleGreater->m_dInput2 = VALUE2;
    
    // calculate the result
    m_pDoubleGreater->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("double greater computation failed", m_bBoolInput() == (VALUE1 > VALUE2));
    
    // set inputs of CDoublGreater
    m_pDoubleGreater->m_dInput1 = VALUE2;
    m_pDoubleGreater->m_dInput2 = VALUE1;

    // calculate the result
    m_pDoubleGreater->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("double greater computation failed", m_bBoolInput() == (VALUE1 < VALUE2));
  // Bouml preserved body end 00082482

} // void CFunctionBlock::TestDoubleGreater()  throw(::std::exception) 




void CFunctionBlock::TestIntGreater()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082502
    const int       VALUE1(rand());
    const int       VALUE2(rand());
    
    
    
    // set inputs of CDoubleEqualOrGreater
    m_pIntGreater->m_iInput1 = VALUE1;
    m_pIntGreater->m_iInput2 = VALUE2;
    
    // calculate the result
    m_pIntGreater->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("integer greater computation failed", m_bBoolInput() == (VALUE1 > VALUE2));
    
    // set inputs of CDoubleEqualOrGreater
    m_pIntGreater->m_iInput1 = VALUE2;
    m_pIntGreater->m_iInput2 = VALUE1;

    // calculate the result
    m_pIntGreater->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("integer greater computation failed", m_bBoolInput() == (VALUE1 < VALUE2));
  // Bouml preserved body end 00082502

} // void CFunctionBlock::TestIntGreater()  throw(::std::exception) 




void CFunctionBlock::TestDoubleArrayAccessor()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082582
    const double            VALUE1(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double            VALUE2(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double            VALUE3(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    ::std::vector< double>  DoubleArray(3);
    
    
    
    // initialise double array with values
    DoubleArray[0]  = VALUE1;
    DoubleArray[1]  = VALUE2;
    DoubleArray[2]  = VALUE3;
    
    // set inputs of CDoublArrayAccessor
    m_pDoubleArrayAccessor->m_Array     = DoubleArray;
    m_pDoubleArrayAccessor->m_iIndex    = 0;
    
    // calculate the result
    m_pDoubleArrayAccessor->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("accessing of first element failed", m_dDoubleInput() == VALUE1);
    
    // set inputs of CDoublArrayAccessor
    m_pDoubleArrayAccessor->m_iIndex    = 1;

    // calculate the result
    m_pDoubleArrayAccessor->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("accessing of second element failed", m_dDoubleInput() == VALUE2);
    
    // set inputs of CDoublArrayAccessor
    m_pDoubleArrayAccessor->m_iIndex    = 2;

    // calculate the result
    m_pDoubleArrayAccessor->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("accessing of third/last element failed", m_dDoubleInput() == VALUE3);
  // Bouml preserved body end 00082582

} // void CFunctionBlock::TestDoubleArrayAccessor()  throw(::std::exception) 




void CFunctionBlock::TestBoolGate()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082602
    // reset boolean input of fixture
    m_bBoolInput = false;
            
    // set inputs of CBoolGate object
    m_pBoolGate->m_bGateInput   = true;
    m_pBoolGate->m_bOpen        = false;
    
    // calculate result
    m_pBoolGate->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("closing of boolean gate failed", m_bBoolInput() == false);
    
    // set inputs of CBoolGate object
    m_pBoolGate->m_bOpen        = true;

    // calculate result
    m_pBoolGate->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("opening of boolean gate failed", m_bBoolInput() == true);
  // Bouml preserved body end 00082602

} // void CFunctionBlock::TestBoolGate()  throw(::std::exception) 




void CFunctionBlock::TestBrakingToTargetSpeed()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082682
    const double                PERMITTED_VELOCITY(100.0);
    const double                VELOCITY(PERMITTED_VELOCITY * 0.90);
    const double                VELOCITY_SOFT(PERMITTED_VELOCITY * 1.025);
    const double                VELOCITY_HARD(PERMITTED_VELOCITY * 1.10);
    const double                VELOCITY_EMERGENCY(PERMITTED_VELOCITY * 2.0);
    const double                ADHESION_FACTOR(1.0);
    const double                ABSOLUTE_DISTANCE(1000.0);
    const double                ABSOLUTE_DISTANCE_TO_PERMITTED(ABSOLUTE_DISTANCE + 500.0);
    
    
    
    // set inputs for permitted velocity
    m_pBrakingToTargetSpeed->m_dCurrentAbsoluteDistance             = ABSOLUTE_DISTANCE;
    m_pBrakingToTargetSpeed->m_dDistanceOfPermittedVelocity         = ABSOLUTE_DISTANCE_TO_PERMITTED;
    m_pBrakingToTargetSpeed->m_dCurrentVelocity                     = VELOCITY;
    m_pBrakingToTargetSpeed->m_dPermittedVelocity                   = PERMITTED_VELOCITY;
    m_pBrakingToTargetSpeed->m_dAdhesionFactor                      = ADHESION_FACTOR;
    
    // calculate outputs
    m_pBrakingToTargetSpeed->Calculate();
    
    // assert results
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake intensity for permitted speed failed", m_dDoubleInput() == 0.0);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation for permitted speed failed", m_bBoolInput() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation flag for permitted speed failed", m_bBoolInput2() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake activation flag soft permitted speed failed", m_bBoolInput3() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of speed warning flag for permitted speed failed", m_bBoolInput4() == false);
    
    
    // set inputs for velocity in soft limit
    m_pBrakingToTargetSpeed->m_dCurrentAbsoluteDistance             = ABSOLUTE_DISTANCE;
    m_pBrakingToTargetSpeed->m_dDistanceOfPermittedVelocity         = ABSOLUTE_DISTANCE_TO_PERMITTED;
    m_pBrakingToTargetSpeed->m_dCurrentVelocity                     = VELOCITY_SOFT;
    m_pBrakingToTargetSpeed->m_dPermittedVelocity                   = PERMITTED_VELOCITY;
    m_pBrakingToTargetSpeed->m_dAdhesionFactor                      = ADHESION_FACTOR;

    // calculate outputs
    m_pBrakingToTargetSpeed->Calculate();

    // assert results
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake intensity for permitted speed failed", m_dDoubleInput() == 0.0);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation for permitted speed failed", m_bBoolInput() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation flag for soft permitted speed failed", m_bBoolInput2() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake activation flag soft permitted speed failed", m_bBoolInput3() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of speed warning flag for permitted speed failed", m_bBoolInput4() == true);
    
    
    // set inputs for velocity over hard limit
    m_pBrakingToTargetSpeed->m_dCurrentAbsoluteDistance             = ABSOLUTE_DISTANCE;
    m_pBrakingToTargetSpeed->m_dDistanceOfPermittedVelocity         = ABSOLUTE_DISTANCE_TO_PERMITTED;
    m_pBrakingToTargetSpeed->m_dCurrentVelocity                     = VELOCITY_HARD;
    m_pBrakingToTargetSpeed->m_dPermittedVelocity                   = PERMITTED_VELOCITY;
    m_pBrakingToTargetSpeed->m_dAdhesionFactor                      = ADHESION_FACTOR;

    // calculate outputs
    m_pBrakingToTargetSpeed->Calculate();

    // assert results
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake intensity for speed over hard limit failed", m_dDoubleInput() >= 1.0 && m_dDoubleInput() <= 100.0);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation for speed over hard limit failed", m_bBoolInput() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation flag for speed over hard limit failed", m_bBoolInput2() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake activation flag for speed over hard limit failed", m_bBoolInput3() == true);
    CPPUNIT_ASSERT_MESSAGE("calculation of speed warning flag for speed over hard limit failed", m_bBoolInput4() == true);
  
    
    // set inputs for velocity over physical/emergency limit
    m_pBrakingToTargetSpeed->m_dCurrentAbsoluteDistance             = ABSOLUTE_DISTANCE;
    m_pBrakingToTargetSpeed->m_dDistanceOfPermittedVelocity         = ABSOLUTE_DISTANCE_TO_PERMITTED;
    m_pBrakingToTargetSpeed->m_dCurrentVelocity                     = VELOCITY_EMERGENCY;
    m_pBrakingToTargetSpeed->m_dPermittedVelocity                   = PERMITTED_VELOCITY;
    m_pBrakingToTargetSpeed->m_dAdhesionFactor                      = ADHESION_FACTOR;

    // calculate outputs
    m_pBrakingToTargetSpeed->Calculate();
    
    // assert results
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake intensity for speed over emergency limit failed", m_dDoubleInput() == 100.0);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation for speed over emergency limit failed", m_bBoolInput() == true);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation flag for speed over emergency limit failed", m_bBoolInput2() == true);
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake activation flag for speed over emergency limit failed", m_bBoolInput3() == true);
    CPPUNIT_ASSERT_MESSAGE("calculation of speed warning flag for speed over emergency limit failed", m_bBoolInput4() == true);
  // Bouml preserved body end 00082682

} // void CFunctionBlock::TestBrakingToTargetSpeed()  throw(::std::exception) 




void CFunctionBlock::TestCeelingSpeedControl()  throw(::std::exception)  
{
  // Bouml preserved body begin 000BCD02
    const double                PERMITTED_VELOCITY(100.0);
    const double                VELOCITY(PERMITTED_VELOCITY * 0.90);
    const double                VELOCITY_SOFT(PERMITTED_VELOCITY * 1.025);
    const double                VELOCITY_HARD(PERMITTED_VELOCITY * 1.10);
    const double                VELOCITY_EMERGENCY(PERMITTED_VELOCITY * 2.10);
    const double                ADHESION_FACTOR(1.0);
    const double                ABSOLUTE_DISTANCE(1000.0);
    
    
    // set inputs for permitted velocity
    m_pCeelingSpeedControl->m_dCurrentAbsoluteDistance  = ABSOLUTE_DISTANCE;
    m_pCeelingSpeedControl->m_dAdhesionFactor           = ADHESION_FACTOR;
    m_pCeelingSpeedControl->m_dPermittedVelocity        = PERMITTED_VELOCITY;
    m_pCeelingSpeedControl->m_dCurrentVelocity          = VELOCITY;

    // calculate outputs
    m_pCeelingSpeedControl->Calculate();
    
    // assert results
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake intensity for permitted speed failed", m_dDoubleInput() == 0.0);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation for permitted speed failed", m_bBoolInput() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation flag for permitted speed failed", m_bBoolInput2() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake activation flag soft permitted speed failed", m_bBoolInput3() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of speed warning flag for permitted speed failed", m_bBoolInput4() == false);
    
    
    // set inputs for velocity in soft limit
    m_pCeelingSpeedControl->m_dCurrentAbsoluteDistance  = ABSOLUTE_DISTANCE;
    m_pCeelingSpeedControl->m_dAdhesionFactor           = ADHESION_FACTOR;
    m_pCeelingSpeedControl->m_dPermittedVelocity        = PERMITTED_VELOCITY;
    m_pCeelingSpeedControl->m_dCurrentVelocity          = VELOCITY_SOFT;

    // calculate outputs
    m_pCeelingSpeedControl->Calculate();

    // assert results
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake intensity for soft permitted speed failed", m_dDoubleInput() == 0.0);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation for soft permitted speed failed", m_bBoolInput() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation flag for soft permitted speed failed", m_bBoolInput2() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake activation flag for soft permitted speed failed", m_bBoolInput3() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of speed warning flag for soft permitted speed failed", m_bBoolInput4() == true);
    
    // set inputs for velocity over hard limit
    m_pCeelingSpeedControl->m_dCurrentAbsoluteDistance  = ABSOLUTE_DISTANCE;
    m_pCeelingSpeedControl->m_dAdhesionFactor           = ADHESION_FACTOR;
    m_pCeelingSpeedControl->m_dPermittedVelocity        = PERMITTED_VELOCITY;
    m_pCeelingSpeedControl->m_dCurrentVelocity          = VELOCITY_HARD;

    // calculate outputs
    m_pCeelingSpeedControl->Calculate();

    // assert results
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake intensity for speed over hard limit failed", m_dDoubleInput() >= 1.0 && m_dDoubleInput() <= 100.0);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation for speed over hard limit failed", m_bBoolInput() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation flag for speed over hard limit failed", m_bBoolInput2() == false);
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake activation flag for speed over hard limit failed", m_bBoolInput3() == true);
    CPPUNIT_ASSERT_MESSAGE("calculation of speed warning flag for speed over hard limit failed", m_bBoolInput4() == true);
    
    // set inputs for velocity over physical/emergency limit
    m_pCeelingSpeedControl->m_dCurrentAbsoluteDistance  = ABSOLUTE_DISTANCE;
    m_pCeelingSpeedControl->m_dAdhesionFactor           = ADHESION_FACTOR;
    m_pCeelingSpeedControl->m_dPermittedVelocity        = PERMITTED_VELOCITY;
    m_pCeelingSpeedControl->m_dCurrentVelocity          = VELOCITY_EMERGENCY;

    // calculate outputs
    m_pCeelingSpeedControl->Calculate();

    // assert results
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake intensity for speed over emergency limit failed", m_dDoubleInput() == 100.0);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation for speed over emergency limit failed", m_bBoolInput() == true);
    CPPUNIT_ASSERT_MESSAGE("calculation of emergency brake activation flag for speed over emergency limit failed", m_bBoolInput2() == true);
    CPPUNIT_ASSERT_MESSAGE("calculation of service brake activation flag for speed over emergency limit failed", m_bBoolInput3() == true);
    CPPUNIT_ASSERT_MESSAGE("calculation of speed warning flag for speed over emergency limit failed", m_bBoolInput4() == true);
  // Bouml preserved body end 000BCD02

} // void CFunctionBlock::TestCeelingSpeedControl()  throw(::std::exception) 




void CFunctionBlock::TestStringGate()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082702
    unsigned int        x(0);
    ::std::string       Value1;



    // create random string
    for (x = 0; x < 20; x++)
    {
        // add random char to string
        Value1 += char(double(rand()) / double(RAND_MAX) * 94.0 + 32.0);

    } // for (x = 0; x < 20; x++)
    
    // reset string input of fixture
    m_StringInput().clear();
            
    // set inputs of CStringGate object
    m_pStringGate->m_GateInput      = Value1;
    m_pStringGate->m_bOpen          = false;
    
    // calculate result
    m_pStringGate->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("closing of string gate failed", m_StringInput().empty());
    
    // set inputs of CStringGate object
    m_pStringGate->m_bOpen        = true;

    // calculate result
    m_pStringGate->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("opening of string gate failed", m_StringInput() == Value1);
  // Bouml preserved body end 00082702

} // void CFunctionBlock::TestStringGate()  throw(::std::exception) 




void CFunctionBlock::TestDoubleGate()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082782
    const double            VALUE1(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double            VALUE2(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    
   
    
    // reset double input of fixture
    m_dDoubleInput = VALUE2;
            
    // set inputs of CDoubleGate object
    m_pDoubleGate->m_dGateInput   = VALUE1;
    m_pDoubleGate->m_bOpen        = false;
    
    // calculate result
    m_pDoubleGate->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("closing of double gate failed", m_dDoubleInput() == VALUE2);
    
    // set inputs of CDoubleGate object
    m_pDoubleGate->m_bOpen        = true;

    // calculate result
    m_pDoubleGate->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("opening of double gate failed", m_dDoubleInput() == VALUE1);
  // Bouml preserved body end 00082782

} // void CFunctionBlock::TestDoubleGate()  throw(::std::exception) 




void CFunctionBlock::TestBoolSwitch()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082802
    // reset boolean input of fixture
    m_bBoolInput = false;
            
    // set inputs of CBoolSwitch object
    m_pBoolSwitch->m_bInput1    = true;
    m_pBoolSwitch->m_bInput2    = false;
    m_pBoolSwitch->m_bUseInput1 = true;
    
    // calculate result
    m_pBoolSwitch->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("activating boolean switch failed", m_bBoolInput() == true);
    
    // set inputs of CBoolSwitch object
    m_pBoolSwitch->m_bUseInput1 = false;

    // calculate result
    m_pBoolSwitch->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("deactivating boolean switch failed", m_bBoolInput() == false);
  // Bouml preserved body end 00082802

} // void CFunctionBlock::TestBoolSwitch()  throw(::std::exception) 




void CFunctionBlock::TestDoubleSwitch()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082882
    const double            VALUE1(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double            VALUE2(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double            VALUE3(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    
    
    
    // reset double input of fixture
    m_dDoubleInput = VALUE3;
            
    // set inputs of CDoubleSwitch object
    m_pDoubleSwitch->m_dInput1      = VALUE1;
    m_pDoubleSwitch->m_dInput2      = VALUE2;
    m_pDoubleSwitch->m_bUseInput1   = true;
    
    // calculate result
    m_pDoubleSwitch->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("activating double switch failed", m_dDoubleInput() == VALUE1);
    
    // set inputs of CDoubleSwitch object
    m_pDoubleSwitch->m_bUseInput1 = false;

    // calculate result
    m_pDoubleSwitch->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("deactivating double switch failed", m_dDoubleInput() == VALUE2);
  // Bouml preserved body end 00082882

} // void CFunctionBlock::TestDoubleSwitch()  throw(::std::exception) 




void CFunctionBlock::TestStringSwitch()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082902
    unsigned int        x(0);
    ::std::string       Value1;
    ::std::string       Value2;



    // create random string
    for (x = 0; x < 20; x++)
    {
        // add random char to strings
        Value1 += char(double(rand()) / double(RAND_MAX) * 94.0 + 32.0);
        Value2 += char(double(rand()) / double(RAND_MAX) * 94.0 + 32.0);

    } // for (x = 0; x < 20; x++)

    // reset string input of fixture
    m_StringInput().clear();
            
    // set inputs of CStringSwitch object
    m_pStringSwitch->m_Input1      = Value1;
    m_pStringSwitch->m_Input2      = Value2;
    m_pStringSwitch->m_bUseInput1   = true;
    
    // calculate result
    m_pStringSwitch->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("activating string switch failed", m_StringInput() == Value1);
    
    // set inputs of CDoubleCStringSwitchSwitch object
    m_pStringSwitch->m_bUseInput1 = false;

    // calculate result
    m_pStringSwitch->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("deactivating string switch failed", m_StringInput() == Value2);
  // Bouml preserved body end 00082902

} // void CFunctionBlock::TestStringSwitch()  throw(::std::exception) 




void CFunctionBlock::TestEnteredTrigger()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082982
    // reset boolean input of fixture
    m_bBoolInput = true;
            
    // set inputs of CEnteredTrigger object
    ::oETCS::DF::CEnteredTrigger::m_bEnteredState   = false;
    
    // calculate result
    m_pEnteredTrigger->Calculate();
    
    // assert result
    CPPUNIT_ASSERT_MESSAGE("setting of entered trigger failed", m_bBoolInput() == false);
    
    // set inputs of CEnteredTrigger object
    ::oETCS::DF::CEnteredTrigger::m_bEnteredState   = true;

    // calculate result
    m_pEnteredTrigger->Calculate();

    // assert result
    CPPUNIT_ASSERT_MESSAGE("setting of entered trigger failed", m_bBoolInput() == true);
  // Bouml preserved body end 00082982

} // void CFunctionBlock::TestEnteredTrigger()  throw(::std::exception) 




void CFunctionBlock::TestStorage()  throw(::std::exception)  
{
  // Bouml preserved body begin 00082A02
    const double                VALUE1(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double                VALUE2(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const double                VALUE3(200.0 * double(rand()) / double(RAND_MAX) - 100.0);
    const int                   VALUE4(rand() - RAND_MAX / 2);
    const ::std::string         VALUE5("3.14");
    ::std::stringstream         Stream;
    ::std::vector< double >     Array(2);
    
    
    
    // set input of CStorage
    m_pStorage->m_dDoubleInput      = VALUE1;
    
    // calculate results
    m_pStorage->Calculate();
    
    // stream value
    Stream.str(::std::string());
    Stream << VALUE1;
            
    // assert results
    CPPUNIT_ASSERT_MESSAGE("copying of double failed", m_dDoubleInput() == VALUE1);
    CPPUNIT_ASSERT_MESSAGE("double conversion to double array failed", m_DoubleArrayInput().size() == 1);
    CPPUNIT_ASSERT_MESSAGE("double conversion to double array failed", m_DoubleArrayInput()[0] == VALUE1);
    CPPUNIT_ASSERT_MESSAGE("double conversion to integer failed", m_iIntInput() == int(VALUE1));
    CPPUNIT_ASSERT_MESSAGE("double conversion to string failed", m_StringInput() == Stream.str());
    CPPUNIT_ASSERT_MESSAGE("double conversion to boolean failed", m_bBoolInput() == bool(VALUE1));
    CPPUNIT_ASSERT_MESSAGE("bit copy of double failed", m_pBitStorage->m_dDoubleInput() == VALUE1);
    
    
    // create array
    Array[0] = VALUE2;
    Array[1] = VALUE3;
    
    // set input of CStorage
    m_pStorage->m_DoubleArrayInput  = Array;
    
    // calculate results
    m_pStorage->Calculate();
    
    // stream value
    Stream.str(::std::string());
    Stream << VALUE2;
    
    // assert results
    CPPUNIT_ASSERT_MESSAGE("double array conversion to double failed", m_dDoubleInput() == VALUE2);
    CPPUNIT_ASSERT_MESSAGE("copying of double array failed",m_DoubleArrayInput().size() == 2);
    CPPUNIT_ASSERT_MESSAGE("copying of double array failed",m_DoubleArrayInput()[0] == VALUE2);
    CPPUNIT_ASSERT_MESSAGE("copying of double array failed",m_DoubleArrayInput()[1] == VALUE3);
    CPPUNIT_ASSERT_MESSAGE("double array conversion to integer failed",m_iIntInput() == int(VALUE2));
    CPPUNIT_ASSERT_MESSAGE("double array conversion to string failed",m_StringInput() == Stream.str());
    CPPUNIT_ASSERT_MESSAGE("copying of double array failed",m_DoubleArrayInput() == Array);
    CPPUNIT_ASSERT_MESSAGE("double array conversion to boolean failed", m_bBoolInput() == bool(VALUE2));
    CPPUNIT_ASSERT_MESSAGE("bit copy of double array failed", m_pBitStorage->m_DoubleArrayInput() == Array);
    
    
    // set input of CStorage
    m_pStorage->m_iIntInput  = VALUE4;

    // calculate results
    m_pStorage->Calculate();

    // stream value
    Stream.str(::std::string());
    Stream << VALUE4;

    // assert results
    CPPUNIT_ASSERT_MESSAGE("integer conversion to double failed", m_dDoubleInput() == double(VALUE4));
    CPPUNIT_ASSERT_MESSAGE("integer conversion to double array failed", m_DoubleArrayInput().size() == 1);
    CPPUNIT_ASSERT_MESSAGE("integer conversion to double array failed", m_DoubleArrayInput()[0] == double(VALUE4));
    CPPUNIT_ASSERT_MESSAGE("copying of integer failed", m_iIntInput() == VALUE4);
    CPPUNIT_ASSERT_MESSAGE("integer conversion to string failed", m_StringInput() == Stream.str());
    CPPUNIT_ASSERT_MESSAGE("integer conversion to boolean failed", m_bBoolInput() == bool(VALUE4));
    CPPUNIT_ASSERT_MESSAGE("bit copy of integer failed", m_pBitStorage->m_iIntInput() == VALUE4);
    
    
    // set input of CStorage
    m_pStorage->m_bBoolInput  = true;

    // calculate results
    m_pStorage->Calculate();
    
    // assert results
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to double failed", m_dDoubleInput() == 1.0);
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to double array failed", m_DoubleArrayInput().size() == 1);
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to double array failed", m_DoubleArrayInput()[0] == 1.0);
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to integer failed", m_iIntInput() == 1);
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to string failed", m_StringInput() == "true");
    CPPUNIT_ASSERT_MESSAGE("copying of boolean failed", m_bBoolInput() == true);
    CPPUNIT_ASSERT_MESSAGE("bit copy of boolean failed", m_pBitStorage->m_bBoolInput() == true);
    
    
    // set input of CStorage
    m_pStorage->m_bBoolInput  = false;

    // calculate results
    m_pStorage->Calculate();

    // assert results
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to double failed", m_dDoubleInput() == 0.0);
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to double array failed", m_DoubleArrayInput().size() == 1);
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to double array failed", m_DoubleArrayInput()[0] == 0.0);
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to integer failed", m_iIntInput() == 0);
    CPPUNIT_ASSERT_MESSAGE("boolean conversion to string failed", m_StringInput() == "false");
    CPPUNIT_ASSERT_MESSAGE("copying of boolean failed", m_bBoolInput() == false);
    CPPUNIT_ASSERT_MESSAGE("bit copy of boolean failed", m_pBitStorage->m_bBoolInput() == false);
    
    
    // set input of CStorage
    m_pStorage->m_StringInput  = VALUE5;

    // calculate results
    m_pStorage->Calculate();

    // assert results
    CPPUNIT_ASSERT_MESSAGE("string conversion to double failed", m_dDoubleInput() == 3.14);
    CPPUNIT_ASSERT_MESSAGE("string conversion to double array failed", m_DoubleArrayInput().size() == 1);
    CPPUNIT_ASSERT_MESSAGE("string conversion to double array failed", m_DoubleArrayInput()[0] == 3.14);
    CPPUNIT_ASSERT_MESSAGE("string conversion to integer failed", m_iIntInput() == 3);
    CPPUNIT_ASSERT_MESSAGE("string conversion to boolean failed", m_bBoolInput() == true);
    CPPUNIT_ASSERT_MESSAGE("copying of string failed", m_StringInput() == VALUE5);
    CPPUNIT_ASSERT_MESSAGE("bit copy of string failed", m_pBitStorage->m_StringInput() == VALUE5);
  // Bouml preserved body end 00082A02

} // void CFunctionBlock::TestStorage()  throw(::std::exception) 




CLanguage::CLanguage()  throw()  
:m_pTelegram(0)
{
  // Bouml preserved body begin 00084702
    // empty constructor
  // Bouml preserved body end 00084702

} // CLanguage::CLanguage()  throw() 




void CLanguage::setUp()  throw(::std::bad_alloc)  
{
  // Bouml preserved body begin 00084602
    // create storages
    m_BoolInput["Storage1"]         = new ::oETCS::DF::BOOL_INPUT_T;
    m_BoolInput["Storage2"]         = new ::oETCS::DF::BOOL_INPUT_T;
    m_BoolInput["Storage3"]         = new ::oETCS::DF::BOOL_INPUT_T;
    m_DoubleInput["Storage1"]       = new ::oETCS::DF::DOUBLE_INPUT_T;
    m_DoubleInput["Storage2"]       = new ::oETCS::DF::DOUBLE_INPUT_T;
    m_DoubleInput["Storage3"]       = new ::oETCS::DF::DOUBLE_INPUT_T;
    m_DoubleArrayInput["Storage1"]  = new ::oETCS::DF::DOUBLE_ARRAY_INPUT_T;
    m_DoubleArrayInput["Storage2"]  = new ::oETCS::DF::DOUBLE_ARRAY_INPUT_T;
    m_DoubleArrayInput["Storage3"]  = new ::oETCS::DF::DOUBLE_ARRAY_INPUT_T;
    m_IntInput["Storage1"]          = new ::oETCS::DF::INT_INPUT_T;
    m_IntInput["Storage2"]          = new ::oETCS::DF::INT_INPUT_T;
    m_IntInput["Storage3"]          = new ::oETCS::DF::INT_INPUT_T;
    m_StringInput["Storage1"]       = new ::oETCS::DF::STRING_INPUT_T;
    m_StringInput["Storage2"]       = new ::oETCS::DF::STRING_INPUT_T;
    m_StringInput["Storage3"]       = new ::oETCS::DF::STRING_INPUT_T;
    
    // create all substructures and connections/flows
    ::oETCS::DF::CStorage*          pStorage1(new ::oETCS::DF::CStorage(&m_StateMachine));
    pStorage1->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(m_BoolInput["Storage1"], &m_StateMachine));
    pStorage1->m_DoubleValue.push_back(new ::oETCS::DF::CFlow< double >(m_DoubleInput["Storage1"], &m_StateMachine));
    pStorage1->m_DoubleArrayValue.push_back(new ::oETCS::DF::CFlow< ::std::vector< double > >(m_DoubleArrayInput["Storage1"], &m_StateMachine));
    pStorage1->m_IntValue.push_back(new ::oETCS::DF::CFlow< signed int >(m_IntInput["Storage1"], &m_StateMachine));
    pStorage1->m_StringValue.push_back(new ::oETCS::DF::CFlow< ::std::string >(m_StringInput["Storage1"], &m_StateMachine));
    ::oETCS::DF::CStorage*          pStorage2(new ::oETCS::DF::CStorage(&m_StateMachine));
    pStorage2->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(m_BoolInput["Storage2"], &m_StateMachine));
    pStorage2->m_DoubleValue.push_back(new ::oETCS::DF::CFlow< double >(m_DoubleInput["Storage2"], &m_StateMachine));
    pStorage2->m_DoubleArrayValue.push_back(new ::oETCS::DF::CFlow< ::std::vector< double > >(m_DoubleArrayInput["Storage2"], &m_StateMachine));
    pStorage2->m_IntValue.push_back(new ::oETCS::DF::CFlow< signed int >(m_IntInput["Storage2"], &m_StateMachine));
    pStorage2->m_StringValue.push_back(new ::oETCS::DF::CFlow< ::std::string >(m_StringInput["Storage2"], &m_StateMachine));
    ::oETCS::DF::CStorage*          pStorage3(new ::oETCS::DF::CStorage(&m_StateMachine));
    pStorage3->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(m_BoolInput["Storage3"], &m_StateMachine));
    pStorage3->m_DoubleValue.push_back(new ::oETCS::DF::CFlow< double >(m_DoubleInput["Storage3"], &m_StateMachine));
    pStorage3->m_DoubleArrayValue.push_back(new ::oETCS::DF::CFlow< ::std::vector< double > >(m_DoubleArrayInput["Storage3"], &m_StateMachine));
    pStorage3->m_IntValue.push_back(new ::oETCS::DF::CFlow< signed int >(m_IntInput["Storage3"], &m_StateMachine));
    pStorage3->m_StringValue.push_back(new ::oETCS::DF::CFlow< ::std::string >(m_StringInput["Storage3"], &m_StateMachine));
    ::oETCS::DF::CVariable*        pVariable0(new ::oETCS::DF::CVariable(&m_StateMachine, 1, ::oETCS::DF::CVariable::INTEGER, 8)); // packet ID
    ::oETCS::DF::CVariable*        pVariable1(new ::oETCS::DF::CVariable(&m_StateMachine, 1, ::oETCS::DF::CVariable::STRING, 96));
                                   pVariable1->m_BitValues.push_back(new ::oETCS::DF::CBitFlow(pStorage1, &m_StateMachine));
    ::oETCS::DF::CVariable*        pVariable2(new ::oETCS::DF::CVariable(&m_StateMachine, 1, ::oETCS::DF::CVariable::INTEGER, 16));
                                   pVariable2->m_BitValues.push_back(new ::oETCS::DF::CBitFlow(pStorage2, &m_StateMachine));
    ::oETCS::DF::CVariable*        pVariable3(new ::oETCS::DF::CVariable(&m_StateMachine, 1, ::oETCS::DF::CVariable::INTEGER, 8));
                                   pVariable3->m_ScaledVariables.push_back(pVariable2);
    ::oETCS::DF::CVariable*        pVariable4(new ::oETCS::DF::CVariable(&m_StateMachine, 1, ::oETCS::DF::CVariable::INTEGER, 8)); // packet ID
    ::oETCS::DF::CVariable*        pVariable5(new ::oETCS::DF::CVariable(&m_StateMachine, 1, ::oETCS::DF::CVariable::KILO_METERS_PER_HOUR, 32));
                                   pVariable5->m_BitValues.push_back(new ::oETCS::DF::CBitFlow(pStorage3, &m_StateMachine));
    ::oETCS::DF::CVariable*        pVariable6(new ::oETCS::DF::CVariable(&m_StateMachine, 1, ::oETCS::DF::CVariable::INTEGER, 8));
                                   pVariable6->m_IteratedVariables.push_back(pVariable5);
    
    ::std::vector< ::oETCS::DF::CVariable* >    Variables1(3);
    ::std::vector< ::oETCS::DF::CVariable* >    Variables2(3);
    Variables1[0] = pVariable0;
    Variables1[1] = pVariable3;
    Variables1[2] = pVariable2;
    Variables2[0] = pVariable4;
    Variables2[1] = pVariable6;
    Variables2[2] = pVariable5;
    
    ::oETCS::DF::CPacket*          pPacket1(new ::oETCS::DF::CPacket(&m_StateMachine, 1, Variables1));
    ::oETCS::DF::CPacket*          pPacket2(new ::oETCS::DF::CPacket(&m_StateMachine, 2, Variables2));
    
    ::std::vector< ::oETCS::DF::CPacket* >      Packets(2);
    Packets[0] = pPacket1;
    Packets[1] = pPacket2;
    
    
    // finally create test telegram with all packets and variables
    m_pTelegram         = new ::oETCS::DF::CTelegram(&m_StateMachine, ::std::vector< ::oETCS::DF::CVariable* >(1, pVariable1), Packets);
  // Bouml preserved body end 00084602

} // void CLanguage::setUp()  throw(::std::bad_alloc) 




void CLanguage::tearDown()  throw()  
{
  // Bouml preserved body begin 00084682
    // nothing to delete explicitly
  // Bouml preserved body end 00084682

} // void CLanguage::tearDown()  throw() 




void CLanguage::TestTelegram()  throw(::std::exception)  
{
  // Bouml preserved body begin 00087B82
    unsigned int                x(0);
    unsigned int                y(0);
    //const char                  pString[] = {'H','e','l','l','o',' ','W','o','r','l','d','!'};
    const unsigned char*        pByte(0);
    const unsigned char         c8ID1(1);
    const unsigned char         c8ID2(2);
    const int                   i8Scalor(3);
    const int                   i16Value(11);
    const int                   i8Iterator(2);
    const int                   i32Iterated1(234);
    const int                   i32Iterated2(567);
    unsigned long               lOffset(0);
    const ::std::string         STRING("Hello World!");
    const unsigned long         BIT_LENGTH((STRING.size() * 8) + 8 + 8 + 8 + 16 + 8 + (32 * 2));
    ::std::vector< bool >       Bits(BIT_LENGTH, 0);
    const int                   SCALED_VALUE(i8Scalor * i16Value);
    const double                ITERATED1(i32Iterated1 / 3.6);
    const double                ITERATED2(i32Iterated2 / 3.6);
    
    
    
    // copy bytes of string into vector
    for (x = 0; x < STRING.size(); x++)
    {
        // initialise byte pointer
        pByte = reinterpret_cast< const unsigned char* >(&STRING[x]);
            
            
        // copy all bits of current byte
        for (y = 0; y < 8; y++)
        {
            // copy current bit
            Bits[(x * 8) + y] = ((1 << (7 - y)) & (*pByte)); 
            
        } // for (y = 0; y < 8; y++)
        
    } // for (x = 0; x < STRING.size(); x++)
    
    // set bit offset
    lOffset += STRING.size() * 8; 
    
    // initialise byte pointer
    pByte = reinterpret_cast< const unsigned char* >(&c8ID1);
    
    // copy all bytes of current byte
    for (y = 0; y < 8; y++)
    {
        // copy current bit
        Bits[lOffset + y] = ((1 << (7 - y)) & (*pByte)); 

    } // for (y = 0; y < 8; y++)
    
    // set bit offset
    lOffset += 8; 
    
    // initialise byte pointer
    pByte = reinterpret_cast< const unsigned char* >(&i8Scalor);

    // copy all bytes of current byte
    for (y = 0; y < 8; y++)
    {
        // copy current bit
        Bits[lOffset + y] = ((1 << (7 - y)) & (*pByte)); 

    } // for (y = 0; y < 8; y++)
    
    // set bit offset
    lOffset += 8;
    
    // initialise byte pointer
    pByte = reinterpret_cast< const unsigned char* >(&i16Value) + 1;
    
    // copy bytes of value into vector
    for (x = 0; x < 2; x++)
    {   
        // copy all bits of current byte
        for (y = 0; y < 8; y++)
        {
            // copy current bit
            Bits[(x * 8) + y + lOffset] = ((1 << (7 - y)) & (*pByte));

        } // for (y = 0; y < 8; y++)

        // increase byte pointer
        pByte--;

    } // for (x = 0; x < 2; x++)
    
    // set bit offset
    lOffset += 16;
    
    // initialise byte pointer
    pByte = reinterpret_cast< const unsigned char* >(&c8ID2);

    // copy all bytes of current byte
    for (y = 0; y < 8; y++)
    {
        // copy current bit
        Bits[lOffset + y] = ((1 << (7 - y)) & (*pByte)); 

    } // for (y = 0; y < 8; y++)

    // set bit offset
    lOffset += 8;
    
    // initialise byte pointer
    pByte = reinterpret_cast< const unsigned char* >(&i8Iterator);

    // copy all bytes of current byte
    for (y = 0; y < 8; y++)
    {
        // copy current bit
        Bits[lOffset + y] = ((1 << (7 - y)) & (*pByte)); 

    } // for (y = 0; y < 8; y++)

    // set bit offset
    lOffset += 8;
    
    // initialise byte pointer
    pByte = reinterpret_cast< const unsigned char* >(&i32Iterated1) + 3;
    
    // copy bytes of iterated into vector
    for (x = 0; x < 4; x++)
    {
        // copy all bits of current byte
        for (y = 0; y < 8; y++)
        {
            // copy current bit
            Bits[(x * 8) + y + lOffset] = ((1 << (7 - y)) & (*pByte)); 

        } // for (y = 0; y < 8; y++)

        // increase byte pointer
        pByte--;

    } // for (x = 0; x < 4; x++)
    
    // set bit offset
    lOffset += 32;
    
    // initialise byte pointer
    pByte = reinterpret_cast< const unsigned char* >(&i32Iterated2) + 3;

    // copy bytes of iterated into vector
    for (x = 0; x < 4; x++)
    {
        // copy all bits of current byte
        for (y = 0; y < 8; y++)
        {
            // copy current bit
            Bits[(x * 8) + y + lOffset] = ((1 << (7 - y)) & (*pByte)); 

        } // for (y = 0; y < 8; y++)

        // increase byte pointer
        pByte--;

    } // for (x = 0; x < 4; x++)

    // set bit offset
    lOffset += 32;
    
    
    // check test preconditon for bit vector
    CPPUNIT_ASSERT_MESSAGE("test precondition bit vector length not correct", lOffset == BIT_LENGTH);
    
    // send bit vector to telegram
    (*m_pTelegram) << Bits;
    
    // check string output of first storage
    CPPUNIT_ASSERT_MESSAGE("string message not corrected dispatched",  (*m_StringInput["Storage1"])() == STRING);
    
    // check integer output of second storage
    CPPUNIT_ASSERT_MESSAGE("scaling not correct applied", (*m_IntInput["Storage2"])() == SCALED_VALUE);
    
    // check double array output of third storage
    CPPUNIT_ASSERT_MESSAGE("size of double array not correct", (*m_DoubleArrayInput["Storage3"])().size() == (unsigned)i8Iterator);
    CPPUNIT_ASSERT_MESSAGE("first iterated value of array false", (*m_DoubleArrayInput["Storage3"])()[0] == ITERATED1);
    CPPUNIT_ASSERT_MESSAGE("first iterated value of array false", (*m_DoubleArrayInput["Storage3"])()[1] == ITERATED2);
  // Bouml preserved body end 00087B82

} // void CLanguage::TestTelegram()  throw(::std::exception) 




CEVCStateMachine::CEVCStateMachine()  throw()  
:m_pStateMachine(0)
{
  // Bouml preserved body begin 00089502
    // empty constructor
  // Bouml preserved body end 00089502

} //  CEVCStateMachine::CEVCStateMachine()  throw() 




void CEVCStateMachine::setUp()  throw(::std::bad_alloc)  
{
  // Bouml preserved body begin 00089582
    ::oETCS::DF::CEVCCondition*                         pFrom1To2(0);
    ::oETCS::DF::CEVCCondition*                         pFrom2To3(0);
    ::oETCS::DF::CStorage*                              pPermanentTrue(0);
    ::std::vector< ::oETCS::DF::CAbstractFlow* >        PermanentTrueOutputs;
    ::oETCS::DF::CLevelCondition*                       pFromSTMToFFN(0);
    ::oETCS::DF::CDMISubject*                           pDMI(0);
    ::std::vector< ::oETCS::DF::CFunctionBlock* >       Flow1STMFBs(0);
    ::std::vector< ::oETCS::DF::CFunctionBlock* >       Flow2STMFBs(0);
    ::std::vector< ::oETCS::DF::CFunctionBlock* >       Flow3FFNFBs(0);
    ::std::vector< ::oETCS::DF::CAbstractFlow* >        Connections1STM(0);
    ::std::vector< ::oETCS::DF::CAbstractFlow* >        Connections2STM(0);
    ::std::vector< ::oETCS::DF::CAbstractFlow* >        Connections3FFN(0);
    ::oETCS::DF::CEVCState*                             pState1(0);
    ::oETCS::DF::CEVCState*                             pState2(0);
    ::oETCS::DF::CEVCState*                             pState3(0);

    
    
    // create EVC state machine object
    m_pStateMachine = new ::oETCS::DF::CEVCStateMachine;
    
    
    // create function blocks for data flows, including EVC conditions
    pFrom1To2       = new ::oETCS::DF::CEVCCondition(m_pStateMachine);
    pFrom2To3       = new ::oETCS::DF::CEVCCondition(m_pStateMachine);
    pFromSTMToFFN   = new ::oETCS::DF::CLevelCondition(m_pStateMachine, "FFN");

    pPermanentTrue  = new ::oETCS::DF::CStorage(m_pStateMachine);
    pPermanentTrue->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(&pFrom1To2->m_bGuard, m_pStateMachine));
    pPermanentTrue->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(&pFrom2To3->m_bGuard, m_pStateMachine));
    pPermanentTrue->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(&pFromSTMToFFN->m_bLevelSwitch, m_pStateMachine));
    pPermanentTrue->m_bBoolInput = true;
    
    
    // create three EVC states
    pState1         = new ::oETCS::DF::CEVCStateMachine::CEVCState(m_pStateMachine);
    pState2         = new ::oETCS::DF::CEVCStateMachine::CEVCState(m_pStateMachine);
    pState3         = new ::oETCS::DF::CEVCStateMachine::CEVCState(m_pStateMachine);


    // create three data flows for EVC states
    Flow1STMFBs.push_back(pPermanentTrue);
    Flow1STMFBs.push_back(pFrom1To2);
    Connections1STM.push_back(pPermanentTrue->m_BoolValue[0]);
    new ::oETCS::DF::CDataFlow(pState1, "STM", Flow1STMFBs, Connections1STM, pDMI);
    Flow2STMFBs.push_back(pPermanentTrue);
    Flow2STMFBs.push_back(pFrom2To3);
    Flow2STMFBs.push_back(pFromSTMToFFN);
    Connections2STM.push_back(pPermanentTrue->m_BoolValue[1]);
    Connections2STM.push_back(pPermanentTrue->m_BoolValue[2]);
    new ::oETCS::DF::CDataFlow(pState2, "STM", Flow2STMFBs, Connections2STM, pDMI);
    new ::oETCS::DF::CDataFlow(pState3, "FFN", Flow3FFNFBs, Connections3FFN, pDMI); 
    

    // create EVC transitions between states
    new ::oETCS::DF::CEVCTransition(pFrom1To2, pState1, pState2, 0);
    new ::oETCS::DF::CEVCTransition(pFrom2To3, pState2, pState3, 1);
  // Bouml preserved body end 00089582

} // void CEVCStateMachine::setUp()  throw(::std::bad_alloc) 




void CEVCStateMachine::tearDown()  throw()  
{
  // Bouml preserved body begin 00089602
    // delete EVC state machine
    delete m_pStateMachine;
  // Bouml preserved body end 00089602

} // void CEVCStateMachine::tearDown()  throw() 




void CEVCStateMachine::TestEVCState()  throw(::std::exception)  
{
  // Bouml preserved body begin 00089682
    decltype (m_pStateMachine->GetStates())                     States(m_pStateMachine->GetStates());
    const ::std::chrono::duration< long long, ::std::micro >    SLEEP_TIME(10 * __DATA_FLOW_SAMPLE_TIME__);
    
    
    // start EVC state machine
    m_pStateMachine->Start(States[0], "STM");
    
    // put thread to sleep to wait for execution of state machine
    ::std::this_thread::sleep_for(SLEEP_TIME);
    
    // check current state and application level
    CPPUNIT_ASSERT_MESSAGE("target state not reached", m_pStateMachine->GetActiveState() == States[2]);
    CPPUNIT_ASSERT_MESSAGE("target application level not reached", m_pStateMachine->m_CurrentApplicationLevel == "FFN");
    
    // finally stop state machine
    m_pStateMachine->Stop();
  // Bouml preserved body end 00089682

} // void CEVCStateMachine::TestEVCState()  throw(::std::exception) 




CControlFlow::CControlFlow()  throw()  
:m_bBoolInput(false),
 m_StringInput(""),
 m_pStateMachine(0),
 m_pControlFlow(0)
{
  // Bouml preserved body begin 00091B82
    // empty constructor
  // Bouml preserved body end 00091B82

} //  CControlFlow::CControlFlow()  throw() 




void CControlFlow::setUp()  throw(::std::bad_alloc)  
{
  // Bouml preserved body begin 00091C02
    ::oETCS::DF::CStorage*                              pPermanentTrue(0);
    ::oETCS::DF::CStorage*                              pPermanentFalse(0);
    ::oETCS::DF::CDMISubject*                           pDMI(0);
    ::std::vector< ::oETCS::DF::CFunctionBlock* >       State1FunctionBlocks(0);
    ::std::vector< ::oETCS::DF::CFunctionBlock* >       State2FunctionBlocks(0);
    ::std::vector< ::oETCS::DF::CFunctionBlock* >       State3FunctionBlocks(0);
    ::std::vector< ::oETCS::DF::CFunctionBlock* >       DataFlowFunctionBlocks(0);
    ::std::vector< ::oETCS::DF::CAbstractFlow* >        FlowsState1(0);
    ::std::vector< ::oETCS::DF::CAbstractFlow* >        FlowsState2(0);
    ::std::vector< ::oETCS::DF::CAbstractFlow* >        FlowsState3(0);
    ::std::vector< ::oETCS::DF::CAbstractFlow* >        Flows(0);
    ::oETCS::DF::CEVCState*                             pEVCState1(0);
    ::oETCS::DF::CControlFlow*                          pControlFlow(0);
    ::oETCS::DF::CControlFlow::CState*                  pState1(0);
    ::oETCS::DF::CControlFlow::CState*                  pState2(0);
    ::oETCS::DF::CControlFlow::CState*                  pState3(0);
    ::oETCS::DF::CCondition*                            pFrom1To2(0);
    ::oETCS::DF::CCondition*                            pFrom2To3(0);
    

    
    
    // create EVC state machine object
    m_pStateMachine     = new ::oETCS::DF::CEVCStateMachine;
    
    
    // create EVC state
    pEVCState1          = new ::oETCS::DF::CEVCStateMachine::CEVCState(m_pStateMachine);
    
    
    // create control flow
    pControlFlow        = new ::oETCS::DF::CControlFlow(m_pStateMachine);
    pControlFlow->m_IsRunning.push_back(new ::oETCS::DF::CFlow< bool >(&m_bBoolInput, m_pStateMachine));
    pControlFlow->m_ActiveStateName.push_back(new ::oETCS::DF::CFlow< ::std::string >(&m_StringInput, m_pStateMachine));
    
    
    // create function blocks for data flow
    pFrom1To2           = new ::oETCS::DF::CCondition(m_pStateMachine);
    pFrom2To3           = new ::oETCS::DF::CCondition(m_pStateMachine);
    pPermanentTrue      = new ::oETCS::DF::CStorage(m_pStateMachine);
    pPermanentTrue->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(&pFrom1To2->m_bGuard, m_pStateMachine));
    pPermanentTrue->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(&pFrom2To3->m_bGuard, m_pStateMachine));
    pPermanentTrue->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(&pControlFlow->m_bStart, m_pStateMachine));
    pPermanentTrue->m_bBoolInput = true;
    pPermanentFalse      = new ::oETCS::DF::CStorage(m_pStateMachine);
    pPermanentFalse->m_BoolValue.push_back(new ::oETCS::DF::CFlow< bool >(&pPermanentTrue->m_bBoolInput, m_pStateMachine));
    pPermanentFalse->m_bBoolInput = false;
    pControlFlow->m_IsRunning.push_back(new ::oETCS::DF::CFlow< bool >(&pPermanentTrue->m_bBoolInput, m_pStateMachine));
    
    
    // create states
    State1FunctionBlocks.push_back(pPermanentTrue);
    State1FunctionBlocks.push_back(pFrom1To2);
    FlowsState1.push_back(pPermanentTrue->m_BoolValue[0]);
    pState1             = new ::oETCS::DF::CControlFlow::CState(pControlFlow, "STATE 1", State1FunctionBlocks, FlowsState1, false, true);
    State2FunctionBlocks.push_back(pPermanentTrue);
    State2FunctionBlocks.push_back(pFrom2To3);
    FlowsState2.push_back(pPermanentTrue->m_BoolValue[1]);
    pState2             = new ::oETCS::DF::CControlFlow::CState(pControlFlow, "STATE 2", State2FunctionBlocks, FlowsState2);
    State3FunctionBlocks.push_back(pPermanentFalse);
    State3FunctionBlocks.push_back(pPermanentTrue);
    FlowsState3.push_back(pPermanentFalse->m_BoolValue[0]);
    pState3             = new ::oETCS::DF::CControlFlow::CState(pControlFlow, "FINAL STATE", State3FunctionBlocks, FlowsState3, true);
    
    
    // create state transitions
    new ::oETCS::DF::CTransition(pEVCState1, pFrom1To2, pState1, pState2, 0);
    new ::oETCS::DF::CTransition(pEVCState1, pFrom2To3, pState2, pState3, 0);
    
    
    // create data flow
    DataFlowFunctionBlocks.push_back(pPermanentTrue);
    DataFlowFunctionBlocks.push_back(pControlFlow);
    Flows.push_back(pPermanentTrue->m_BoolValue[2]);
    Flows.push_back(pControlFlow->m_ActiveStateName[0]);
    new ::oETCS::DF::CEVCStateMachine::CEVCState::CDataFlow(pEVCState1, "IGNORE", DataFlowFunctionBlocks, Flows, pDMI);
  // Bouml preserved body end 00091C02

} // void CControlFlow::setUp()  throw(::std::bad_alloc) 




void CControlFlow::tearDown()  throw()  
{
  // Bouml preserved body begin 00091C82
    // delete EVC state machine
    delete m_pStateMachine;
  // Bouml preserved body end 00091C82

} // void CControlFlow::tearDown()  throw() 




void CControlFlow::TestState()  throw(::std::exception)  
{
  // Bouml preserved body begin 00091D02
    decltype (m_pStateMachine->GetStates())                     States(m_pStateMachine->GetStates());
    const ::std::chrono::duration< long long, ::std::micro >    SLEEP_TIME(10 * __DATA_FLOW_SAMPLE_TIME__);
        
    
        
    // check, number of available states in EVC state machine
    CPPUNIT_ASSERT_MESSAGE("no states available in EVC state machine", States.size() > 0);
    
    // start state machine
    m_pStateMachine->Start(States[0], "IGNORE");
    
    // put thread to sleep to wait for execution of state machine
    ::std::this_thread::sleep_for(SLEEP_TIME);
    
    // check string input for final state name
    CPPUNIT_ASSERT_MESSAGE("string input of last state name not correct", m_StringInput() == "FINAL STATE");
    
    // check boolean input for running control flow
    CPPUNIT_ASSERT_MESSAGE("boolean input for running control flow not correct", m_bBoolInput() == false);
    
    // finally stop state machine
    m_pStateMachine->Stop();
  // Bouml preserved body end 00091D02

} // void CControlFlow::TestState()  throw(::std::exception) 




void CGOPPRRTransformer::setUp()  throw(::std::bad_alloc)  
{
  // Bouml preserved body begin 00043A82
    // create test objects
    this->m_pSyntaxLexer = new ::DSM::CGOPPRRTransformer();
    this->m_pChecker     = new ::GOPPRR::CConstraintChecker;
  // Bouml preserved body end 00043A82

} // void CGOPPRRTransformer::setUp()  throw(::std::bad_alloc) 




void CGOPPRRTransformer::tearDown()  throw()  
{
  // Bouml preserved body begin 00043B02
    // delete test objects
    delete this->m_pSyntaxLexer;
    delete this->m_pChecker;
  // Bouml preserved body end 00043B02

} // void CGOPPRRTransformer::tearDown()  throw() 




void CGOPPRRTransformer::TestCreateSyntax()  throw(::std::exception)  
{
  // Bouml preserved body begin 00043B82
    ::GOPPRR::CProject*          pProject(0);
    ::GOPPRR::CGraph*            pRootGraph(0);
    ::DSM::CGOPPRRSyntaxTree*    pSyntaxTree(0);
    
        
    // execute CreateSyntax() on test object
    m_pSyntaxLexer->CreateSyntaxFromFile("openETCS.xml", pRootGraph);
    
    // get project from lexer
    pSyntaxTree = m_pSyntaxLexer->GetGOPRRTree();

    // assert tree pointer
    CPPUNIT_ASSERT(pSyntaxTree != 0);

    // get pointer to project from tree
    pProject = pSyntaxTree->GetProject();

    // assert project pointer
    CPPUNIT_ASSERT_MESSAGE("no project object created", pProject != 0);

    // assert, if a root graph was created
    CPPUNIT_ASSERT_MESSAGE("no root graph created", pRootGraph != 0);
    
    // assert of identity of root graph
    CPPUNIT_ASSERT_MESSAGE("root graph not identical", &(pProject->m_GraphSet[ pRootGraph->m_OID ]) == pRootGraph);
    
    // call generated function from MetaEdit+ to execute all asserts
    ::MetaEdit::ExecuteAll(pProject);
  // Bouml preserved body end 00043B82

} // void CGOPPRRTransformer::TestCreateSyntax()  throw(::std::exception) 




void CGOPPRRTransformer::TestStaticSemantics()  throw(::std::exception)  
{
  // Bouml preserved body begin 00103002
    ::GOPPRR::CProject*          pProject(0);
    ::GOPPRR::CGraph*            pRootGraph(0);
    ::DSM::CGOPPRRSyntaxTree*    pSyntaxTree(0);


    // execute CreateSyntax() on test object
    m_pSyntaxLexer->CreateSyntaxFromFile("openETCS.xml", pRootGraph);

    // get project from lexer
    pSyntaxTree = m_pSyntaxLexer->GetGOPRRTree();

    // assert tree pointer
    CPPUNIT_ASSERT(pSyntaxTree != 0);

    // get pointer to project from tree
    pProject = pSyntaxTree->GetProject();

    // assert project pointer
    CPPUNIT_ASSERT_MESSAGE("no project object created", pProject != 0);

    // assert, if a root graph was created
    CPPUNIT_ASSERT_MESSAGE("no root graph created", pRootGraph != 0);
    
    // implement constraint checking
    CPPUNIT_ASSERT_MESSAGE("static semantics violated", m_pChecker->Check(pProject, "", nullptr));
  // Bouml preserved body end 00103002

} // void CGOPPRRTransformer::TestStaticSemantics()  throw(::std::exception) 




void CGenerator::setUp()  throw(::std::bad_alloc)  
{
  // Bouml preserved body begin 000E9082
    // create test objects
    m_pCPPGenerator = new ::oETCS::GEN::CCPPGenerator;
  // Bouml preserved body end 000E9082

} // void CGenerator::setUp()  throw(::std::bad_alloc) 




void CGenerator::tearDown()  throw()  
{
  // Bouml preserved body begin 000E9102
    // delete test objects
    delete this->m_pCPPGenerator;
  // Bouml preserved body end 000E9102

} // void CGenerator::tearDown()  throw() 




void CGenerator::TestGenerateCpp()  throw(::std::exception)  
{
  // Bouml preserved body begin 000E9182
    ::GOPPRR::CGraph*                                       pRootGraph(nullptr);
    ::GOPPRR::CProject*                                     pProject(nullptr);
    ::DSM::CGOPPRRTransformer                               Transformer;
    ::DSM::CSyntaxTree*                                     pTree(nullptr);
    ::std::stringstream                                     GeneratedCpp;
    ::std::regex                                            InstanceRegex;
    ::std::smatch                                           Results;
    ::std::string                                           Expression;
    ::std::string                                           Cpp;
    const ::std::regex                                      FB_TYPE_REGEX("(AND)|(OR)|(XOR)|(NOT)|(Sum)|(Subtraction)|(Division)"
                                                                          "|(Multiplication)|(DoubleEqual)|(IntEqual)|(StringEqual)"
                                                                          "|(DoubleEqualOrGreater)|(DoubleGreater)|(IntGreater)"
                                                                          "|(DoubleArrayAccessor)|(VariableStorage)|(Odometer)"
                                                                          "|(ServiceBrake)|(EmergencyBrake)|(BrakingToTargetSpeed)"
                                                                          "|(CeelingSpeedControl)|(CommunicationReader)|(CommunicationSender)"
                                                                          "|(BoolGate)|(DoubleGate)|(StringGate)|(BoolSwitch)|(DoubleSwitch)"
                                                                          "|(StringSwitch)|(ModeGuard)|(ApplicationLevelType)|(DMIOutput)"
                                                                          "|(DMIInput)|(EnteredTrigger)|(EmbeddedStateMachine)|(StateGuard)");
    const ::std::regex                                      DATAFLOW_TYPE_REGEX("DataFlow");
    const ::std::regex                                      EVC_STATE_TYPE_REGEX("Mode");
    const ::std::regex                                      VARIABLE_TYPE_REGEX("VariableInstance");
    const ::std::regex                                      PACKET_TYPE_REGEX("Packet");
    const ::std::regex                                      TELEGRAM_TYPE_REGEX("Telegram");
    const ::std::regex                                      BALISEREADER_TYPE_REGEX("BaliseReader");
    const ::std::regex                                      BALISESENDER_TYPE_REGEX("BaliseSender");
    decltype (pProject->m_ObjectSet.begin())                o;
    decltype (pProject->m_RelationshipSet.begin())          r;
    bool                                                    bFound(false);
    
    
    
    // create transformer object
    
    // generate C++ source from XML
    Transformer.CreateSyntaxFromFile("openETCS.xml", pRootGraph);
    
    // get syntax tree from transformer
    pTree = Transformer.GetTree();
    
    // get project from tree
    pProject = static_cast< ::DSM::CGOPPRRSyntaxTree* >(pTree)->GetProject();
    
    // generate C++ source into string stream
    m_pCPPGenerator->Generate(pTree, GeneratedCpp);
    
    // get string from stringstream
    Cpp = GeneratedCpp.str();
    
    
    // process all objects in project
    for (o = pProject->m_ObjectSet.begin(); o != pProject->m_ObjectSet.end(); o++)
    {
        // check type of current object
        if (::std::regex_match(o->second.m_Type, FB_TYPE_REGEX))
        {
            // create regular expression for current FB instance
            Expression = ".*\n::oETCS::DF::C.+ +FBO_" + o->second.m_OID + "\\(&EVC\\);.*";
            
            // create a regular expression type from string expression
            InstanceRegex  = ::std::regex(Expression);

            // search stream for instance declaration
            bFound = ::std::regex_match(Cpp, Results, InstanceRegex); 

            // assert existence of current object
            CPPUNIT_ASSERT_MESSAGE("function block object declaration not found, OID: " + o->second.m_OID, bFound == true);

            // assert existence of multiple object instances
            CPPUNIT_ASSERT_MESSAGE("function block object declaration found multiple times, OID: " + o->second.m_OID, Results.size() == 1);
            
        } // if (::std::regex_match(o->second.m_Type, FB_TYPE_REGEX))
        else if (::std::regex_match(o->second.m_Type, EVC_STATE_TYPE_REGEX))
        {
            // create regular expression for current mode instance
            Expression = ".*\n::oETCS::DF::CEVCState +EVCState_" + o->second.m_OID + "\\(&EVC\\);.*";
            
            // create a regular expression type from string expression
            InstanceRegex  = ::std::regex(Expression);

            // search stream for instance declaration
            bFound = ::std::regex_match(Cpp, Results, InstanceRegex);

            // assert existence of current object
            CPPUNIT_ASSERT_MESSAGE("EVC mode object declaration not found, OID: " + o->second.m_OID, bFound == true);

            // assert existence of multiple object instances
            CPPUNIT_ASSERT_MESSAGE("EVC mode object declaration found multiple times, OID: " + o->second.m_OID, Results.size() == 1);
            
        } // else if (::std::regex_match(o->second.m_Type, EVC_STATE_TYPE_REGEX))
        else if (::std::regex_match(o->second.m_Type, VARIABLE_TYPE_REGEX))
        {
            // create regular expression for current mode instance
            Expression = ".*\n::oETCS::DF::CVariable +VAR_" + o->second.m_OID + "\\(&EVC,.+\\);.*";
            
            // create a regular expression type from string expression
            InstanceRegex  = ::std::regex(Expression);

            // search stream for instance declaration
            bFound = ::std::regex_match(Cpp, Results, InstanceRegex);

            // assert existence of current object
            CPPUNIT_ASSERT_MESSAGE("language variable object declaration not found, OID: " + o->second.m_OID, bFound == true);

            // assert existence of multiple object instances
            CPPUNIT_ASSERT_MESSAGE("language variable object declaration found multiple times, OID: " + o->second.m_OID, Results.size() == 1);
            
        } // else if (::std::regex_match(o->second.m_Type, VARIABLE_TYPE_REGEX))
        else if (::std::regex_match(o->second.m_Type, PACKET_TYPE_REGEX))
        {
            // create regular expression for current mode instance
            Expression = ".*\n::oETCS::DF::CPacket +PACK_" + o->second.m_OID + "\\(&EVC,.+\\);.*";

            // create a regular expression type from string expression
            InstanceRegex  = ::std::regex(Expression);

            // search stream for instance declaration
            bFound = ::std::regex_match(Cpp, Results, InstanceRegex);

            // assert existence of current object
            CPPUNIT_ASSERT_MESSAGE("language packet object declaration not found, OID: " + o->second.m_OID, bFound == true);

            // assert existence of multiple object instances
            CPPUNIT_ASSERT_MESSAGE("language packet object declaration found multiple times, OID: " + o->second.m_OID, Results.size() == 1);

        } // else if (::std::regex_match(o->second.m_Type, PACKET_TYPE_REGEX))
        else if (::std::regex_match(o->second.m_Type, TELEGRAM_TYPE_REGEX))
        {
            // create regular expression for current mode instance
            Expression = ".*\n::oETCS::DF::CTelegram +TEL_" + o->second.m_OID + "_._.+\\(&EVC,.+\\);.*";

            // create a regular expression type from string expression
            InstanceRegex  = ::std::regex(Expression);

            // search stream for instance declaration
            bFound = ::std::regex_match(Cpp, Results, InstanceRegex);

            // assert existence of current object
            CPPUNIT_ASSERT_MESSAGE("language telegram object declaration not found, OID: " + o->second.m_OID, bFound == true);

            // assert existence of multiple object instances
            CPPUNIT_ASSERT_MESSAGE("language telegram object declaration found multiple times, OID: " + o->second.m_OID, Results.size() == 1);

        } // else if (::std::regex_match(o->second.m_Type, TELEGRAM_TYPE_REGEX))
        else if (::std::regex_match(o->second.m_Type, BALISEREADER_TYPE_REGEX))
        {
            // create regular expression for current mode instance
            Expression = ".*\n::oETCS::DF::CBaliseDeviceIn +BALIN_" + o->second.m_OID + "_._.+\\(.+\\);.*";

            // create a regular expression type from string expression
            InstanceRegex  = ::std::regex(Expression, ::std::regex_constants::extended);

            // search stream for instance declaration
            bFound = ::std::regex_match(Cpp, Results, InstanceRegex);

            // assert existence of current object
            CPPUNIT_ASSERT_MESSAGE("balise reader object declaration not found, OID: " + o->second.m_OID, bFound == true);

        } // else if (::std::regex_match(o->second.m_Type, BALISEREADER_TYPE_REGEX))
        else if (::std::regex_match(o->second.m_Type, BALISESENDER_TYPE_REGEX))
        {
            // create regular expression for current mode instance
            Expression = ".*\n::oETCS::DF::CBaliseDeviceIn +BALOUT_" + o->second.m_OID + "_._.+\\(.+\\);.*";

            // create a regular expression type from string expression
            InstanceRegex  = ::std::regex(Expression, ::std::regex_constants::extended);

            // search stream for instance declaration
            bFound = ::std::regex_match(Cpp, Results, InstanceRegex);

            // assert existence of current object
            CPPUNIT_ASSERT_MESSAGE("balise sender object declaration not found, OID: " + o->second.m_OID, bFound == true);

        } // else if (::std::regex_match(o->second.m_Type, BALISESENDER_TYPE_REGEX))

    } // for (o = pProject->m_ObjectSet.begin(); o != pProject->m_ObjectSet.end(); o++)
    

//  WARNING: data flow chekcing currently disabled due to problem with telegram flows in commnunication reader ander sender graphs
//  // process all relationships in project
//    for (r = pProject->m_RelationshipSet.begin(); r != pProject->m_RelationshipSet.end(); r++)
//    {
//        // check type of current relationship
//        if (::std::regex_match(r->second.m_Type, DATAFLOW_TYPE_REGEX))
//        {
//            // create regular expression for current mode instance
//            Expression = ".*\n::oETCS::DF::(CBitFlow|CFlow< .+ >) +FLOW_" + r->second.m_OID + "\\(.+\\);.*";
//
//            // create a regular expression type from string expression
//            InstanceRegex  = ::std::regex(Expression, ::std::regex_constants::extended);
//
//            // search stream for instance declaration
//            bFound = ::std::regex_match(Cpp, Results, InstanceRegex);
//
//            // assert existence of current object
//            CPPUNIT_ASSERT_MESSAGE("flow object (from relationship) declaration not found, OID: " + r->second.m_OID, bFound == true);
//
//        } // if (::std::regex_match(r->second.m_Type, DATAFLOW_TYPE_REGEX))
//
//    } // for (r = pProject->m_RelationshipSet.begin(); r != pProject->m_RelationshipSet.end(); r++)
    
  // Bouml preserved body end 000E9182

} // void CGenerator::TestGenerateCpp()  throw(::std::exception) 






} // namespace oETCS::UT

} // namespace oETCS
